import decimal
import sys
import csv
from PyQt5.QtWidgets import *
from PyQt5 import uic, QtGui
from PyQt5 import QtCore
from PyQt5.QtCore import *
import numpy as np
from datetime import datetime
import os
from mysqlquerys import connect


class MyApp(QMainWindow):
    def __init__(self):
        super(MyApp, self).__init__()
        path2src, pyFileName = os.path.split(__file__)
        uiFileName = 'gui.ui'
        path2GUI = os.path.join(path2src, 'GUI', uiFileName)
        Ui_MainWindow, QtBaseClass = uic.loadUiType(path2GUI)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        ini_file = r"D:\Python\MySQL\database.ini"
        self.db = connect.DbConnection(ini_file)

        self.ui.GB_tablesList.setVisible(False)
        self.ui.tableTab.setVisible(False)
        self.populate_listWidgetDataBases()
        self.ui.listWidgetDataBases.itemDoubleClicked.connect(self.setActiveDataBase)
        self.ui.listWidgetTables.itemDoubleClicked.connect(self.setActiveTable)
        self.ui.PBDefaultFilter.clicked.connect(self.populateTableWidget)
        self.ui.pbImportCSV.clicked.connect(self.importCSV)
        self.ui.pbExportCSV.clicked.connect(self.exportCSV)

        self.ui.tableWidget.horizontalHeader().sectionClicked.connect(self.sort)
        self.ui.tableWidget.horizontalHeader().sectionDoubleClicked.connect(self.setFilter)

        self.ui.pbEditTable.clicked.connect(self.editTable)
        self.ui.PBEditTableColsProps.clicked.connect(self.editTableColsProps)
        self.ui.PBSaveTableColsProps.clicked.connect(self.saveEditedValuesColsProps)

        self.ui.pushButtonCreateTable.clicked.connect(self.createTable)
        self.ui.pbCreateDB.clicked.connect(self.createDataBase)
        self.ui.pbAddRow.clicked.connect(self.addRow)
        self.ui.pbSave.clicked.connect(self.saveEditedValues)
        self.ui.listWidgetDataBases.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.ui.listWidgetDataBases.customContextMenuRequested.connect(self.contextListDataBases)

        self.ui.listWidgetTables.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.ui.listWidgetTables.customContextMenuRequested.connect(self.contextListTables)
        self.ui.tableTab.currentChanged.connect(self.showColsProps)

        vHeader = self.ui.tableWidget.verticalHeader()
        vHeader.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        vHeader.customContextMenuRequested.connect(self.contextVHeader)

        vHeader = self.ui.tableColsProps.verticalHeader()
        vHeader.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        vHeader.customContextMenuRequested.connect(self.contextVHeaderTableColsProps)
        self.editModeTableColsProps = False

    def contextVHeaderTableColsProps(self, event):
        print(sys._getframe().f_code.co_name)
        if self.editModeTableColsProps:
            item = self.ui.tableColsProps.itemAt(event)
            contextMenu = QMenu(self)
            if item is not None:
                contextMenu.addAction('Insert Column(s)')
                contextMenu.addAction('Delete Column(s)')
                #
                # print(selectedRowsIndexes)
            elif item is None:
                contextMenu.addAction('AddColumn(s)')

            action = contextMenu.exec_(self.ui.tableColsProps.mapToGlobal(event))
            if action is not None:
                if action.text() == 'Insert Column(s)':
                    selectedRowsIndexes = self.ui.tableColsProps.selectionModel().selectedRows()
                    newCols = CreateTable.createColumnQuery(self.activeTable.table_name)
                    indxs = []
                    for indx in selectedRowsIndexes:
                        indxs.append(indx.row())
                    position2Insert = min(indxs)
                    for newCol in newCols:
                        name, type, lengthVal = newCol
                        self.activeTable.addColumn(name, type, lengthVal, position2Insert)
                        position2Insert += 1
                elif action.text() == 'AddColumn(s)':
                    newCols = CreateTable.createColumnQuery(self.activeTable.table_name)
                    if newCols:
                        for newCol in newCols:
                            name, typeCol, lengthVal = newCol
                            self.activeTable.addColumn(name, typeCol, lengthVal, None)
                elif action.text() == 'Delete Column(s)':
                    selectedColumnsIndexes = self.ui.tableColsProps.selectionModel().selectedRows()
                    cols2Del = []
                    for indx in selectedColumnsIndexes:
                        cols2Del.append(self.activeTable.columnsNames[indx.row()])

                    for col in cols2Del:
                        self.activeTable.dropColumn(col)
                self.showColsProps()

    def editTableColsProps(self):
        print(sys._getframe().f_code.co_name)
        self.editModeTableColsProps = True
        self.ui.tableColsProps.setEditTriggers(QTableWidget.AllEditTriggers)
        self.ui.tableColsProps.cellChanged.connect(self.bufferChangesColsProps)
        # self.ui.tableColsProps.currentCellChanged.connect(self.bufferChangesColsProps)

    def bufferChangesColsProps(self, row, column):
        print(sys._getframe().f_code.co_name)
        if self.editModeTableColsProps:
            cell = self.ui.tableColsProps.item(row, column)
            if cell:
                val2Moify = cell.text()
            prop2Modify = self.ui.tableColsProps.horizontalHeaderItem(column).text()
            if prop2Modify == 'ColName':
                oldColName = self.activeTable.columnsNames[row]
                tup = ('renameColumn', oldColName, val2Moify)
                self.colsPropsEditArr.append(tup)
            elif prop2Modify == 'Type':
                colName = self.activeTable.columnsNames[row]
                tup = ('modifyColType', colName, val2Moify)
                self.colsPropsEditArr.append(tup)
            elif prop2Modify == 'Null':
                colName = self.activeTable.columnsNames[row]
                colTypee = self.ui.tableColsProps.item(row, 1).text()
                colVals = self.activeTable.returnColumn(colName)
                cell = self.ui.tableColsProps.item(row, column)
                if cell.checkState() == QtCore.Qt.Checked:
                    null = True
                elif cell.checkState() == QtCore.Qt.Unchecked:
                    null = False
                    if None in colVals:
                        message = 'Column has empty cells already'
                        QMessageBox.warning(self, 'Missing Data', message, QMessageBox.Ok)
                        cell.setCheckState(QtCore.Qt.Checked)
                        return
                tup = ('modifyColNullConstrain', colName, colTypee, null)
                self.colsPropsEditArr.append(tup)
            elif prop2Modify == 'Key':
                colName = self.activeTable.columnsNames[row]
                widget = self.ui.tableColsProps.cellWidget(row, column)
                keyType = widget.currentText()
                if keyType == 'NONE':
                    key = None
                    refTable = None
                    refColumn = None
                elif keyType == 'PRI':
                    key = 'PRI'
                    refTable = None
                    refColumn = None
                elif keyType == 'FOREIGN':
                    key = 'FOREIGN'
                    refTable, okPress = QInputDialog.getItem(self, 'Select Reference Table', 'Table',
                                                             self.activeDataBase.tables, 0, False)
                    rtab = connect.Table(self.db.data_base_type, self.activeDataBase.data_base_name, refTable)
                    refColumn, okPress = QInputDialog.getItem(self, 'Select Reference Table', 'Table',
                                                              rtab.columnsNames, 0, False)
                tup = ('modifyColKey', colName, key, refTable, refColumn)
                self.colsPropsEditArr.append(tup)
            elif prop2Modify == 'Default':
                colName = self.activeTable.columnsNames[row]
                colTypee = self.ui.tableColsProps.item(row, 1).text()
                tup = ('colDefaultVal', colName, colTypee, val2Moify)
                self.colsPropsEditArr.append(tup)
            elif prop2Modify == 'AutoIncrement':
                colName = self.activeTable.columnsNames[row]
                colTypee = self.ui.tableColsProps.item(row, 1).text()
                cell = self.ui.tableColsProps.item(row, column)
                if cell.checkState() == QtCore.Qt.Checked:
                    autoIncrement = True
                elif cell.checkState() == QtCore.Qt.Unchecked:
                    autoIncrement = False
                tup = ('modify2AutoIncrement', colName, colTypee, autoIncrement)
                self.colsPropsEditArr.append(tup)

    def saveEditedValuesColsProps(self):
        print(sys._getframe().f_code.co_name)
        for edit in self.colsPropsEditArr:
            definition, args = edit[0], edit[1:]
            getattr(self.activeTable, definition)(*args)
        self.colsPropsEditArr = []

    def bufferChanges(self, row, column):
        print(sys._getframe().f_code.co_name)
        if self.editMode:
            cell = self.ui.tableWidget.item(row, column)
            val2Moify = cell.text()
            column2Modify = self.activeTable.columnsNames[column]
            cell = self.ui.tableWidget.item(row, 0)
            refValue = cell.text()
            refColumn = self.activeTable.columnsNames[0]
            tup = (column2Modify, val2Moify, refColumn, refValue)
            self.editArr.append(tup)

    def saveEditedValues(self):
        for edit in self.editArr:
            column2Modify, val2Moify, refColumn, refValue = edit
            self.activeTable.changeCellContent(column2Modify, val2Moify, refColumn, refValue)

    def populate_listWidgetDataBases(self):
        print(sys._getframe().f_code.co_name)
        self.ui.listWidgetDataBases.clear()
        for i, db in enumerate(self.db.databases):
            self.ui.listWidgetDataBases.addItem(str(db))

    def setActiveDataBase(self, item):
        print(sys._getframe().f_code.co_name)
        self.ui.GB_tablesList.setVisible(True)
        self.db.active_data_base = item.text()
        self.activeDataBase = self.db.active_data_base
        self.populate_listWidgetTables()

    def populate_listWidgetTables(self):
        print(sys._getframe().f_code.co_name)
        self.ui.listWidgetTables.clear()
        for i, tab in enumerate(self.activeDataBase.tables):
            self.ui.listWidgetTables.addItem(str(tab))

    def setActiveTable(self, item):
        print(sys._getframe().f_code.co_name)
        self.ui.tableTab.setVisible(True)
        self.activeDataBase.active_table = item.text()
        self.activeTable = self.activeDataBase.active_table
        self.defaultFilter = True
        self.filterList = []
        self.editMode = False
        self.editArr = []
        self.colsPropsEditArr = []
        self.populateTableWidget()

    def populateTableWidget(self):
        print(sys._getframe().f_code.co_name)
        self.ui.tableTab.setTabText(0, self.activeTable.table_name)
        self.ui.tableTab.setTabText(1, 'Data Type')
        self.ui.tableTab.setCurrentIndex(0)

        self.data = np.atleast_2d(self.activeTable.data)
        if self.data.shape == (1, 0):
            self.data = np.empty((0, len(self.activeTable.columnsNames)))

        header = self.ui.tableWidget.horizontalHeader()
        header.setSectionResizeMode(QHeaderView.ResizeToContents)
        self.ui.tableWidget.setEditTriggers(QTableWidget.NoEditTriggers)
        self.populateTable()

    def populateTable(self):
        print(sys._getframe().f_code.co_name)
        self.ui.tableWidget.clear()
        self.ui.tableWidget.setColumnCount(len(self.activeTable.columnsNames))
        self.ui.tableWidget.setHorizontalHeaderLabels(self.activeTable.columnsNames)
        self.ui.tableWidget.setRowCount(self.data.shape[0])
        for col in range(self.data.shape[1]):
            headerName = self.ui.tableWidget.horizontalHeaderItem(col).text()
            for row in range(self.data.shape[0]):
                if isinstance(self.data[row, col], int):
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, self.data[row, col])
                elif isinstance(self.data[row, col], decimal.Decimal):
                    val = float(self.data[row, col])
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, val)
                elif headerName == 'path':
                    path = self.data[row, col]
                    if path:
                        url = bytearray(QUrl.fromLocalFile(path).toEncoded()).decode()
                        item = QLabel()
                        text = "<a href={}>Link</a>".format(url)
                        item.setText(text)
                        item.setOpenExternalLinks(True)
                        self.ui.tableWidget.setCellWidget(row, col, item)
                        continue
                    else:
                        item = QTableWidgetItem(str(self.data[row, col]))
                else:
                    item = QTableWidgetItem(str(self.data[row, col]))
                self.ui.tableWidget.setItem(row, col, item)

    def editTable(self):
        print(sys._getframe().f_code.co_name)
        self.editMode = True
        self.ui.tableWidget.setEditTriggers(QTableWidget.AllEditTriggers)
        self.ui.tableWidget.cellChanged.connect(self.bufferChanges)

    def addRow(self):
        print(sys._getframe().f_code.co_name)
        colsProps = self.activeTable.columnsProperties
        for k, v in colsProps.items():
            colType, null, key, default, extra = v
            if 'auto_increment' in extra:
                v.append(self.activeTable.noOfRows + 1)

        res = InsertNewRow.getNewRowValues(colsProps)
        if res:
            cols, values = res
            if 0 < len(cols) == len(values) > 0:
                self.activeTable.add_row(cols, values)
        self.populateTableWidget()

    def readTable(self):
        print(sys._getframe().f_code.co_name)
        array = np.empty([self.ui.tableWidget.rowCount(), self.ui.tableWidget.columnCount()], dtype=object)
        for col in range(self.ui.tableWidget.columnCount()):
            for row in range(self.ui.tableWidget.rowCount()):
                cell = self.ui.tableWidget.item(row, col)
                if cell.text():
                    array[row, col] = cell.text()
                else:
                    array[row, col] = ''
        return array

    def contextListDataBases(self, event):
        print(sys._getframe().f_code.co_name)
        contextMenu = QMenu(self)
        item = self.ui.listWidgetDataBases.itemAt(event)
        if item is not None:
            contextMenu.addAction('Drop')
        else:
            contextMenu.addAction('new schema')

        action = contextMenu.exec_(self.ui.listWidgetDataBases.mapToGlobal(event))

        if action is not None:
            if action.text() == 'Drop':
                self.db.drop_data_base(item.text())
                self.populate_listWidgetDataBases()
            elif action.text() == 'new schema':
                self.createDataBase()

    def contextListTables(self, event):
        print(sys._getframe().f_code.co_name)
        contextMenu = QMenu(self)
        item = self.ui.listWidgetTables.itemAt(event)
        if item is not None:
            contextMenu.addAction('Drop')
            contextMenu.addAction('Rename')
            contextMenu.addAction('Delete all data')
        else:
            contextMenu.addAction('new table')

        action = contextMenu.exec_(self.ui.listWidgetTables.mapToGlobal(event))

        if action is not None:
            if action.text() == 'Drop':
                self.activeDataBase.drop_table(item.text())
                self.populate_listWidgetTables()
            elif action.text() == 'new table':
                self.createTable()
            elif action.text() == 'Delete all data':
                self.activeDataBase.deleteAllRows(item.text())
                self.populate_listWidgetTables()
            elif action.text() == 'Rename':
                text, okPressed = QInputDialog.getText(self, "Get text", "New Table Name:", QLineEdit.Normal, "")
                if okPressed and text != '':
                    self.activeDataBase.renameTable(item.text(), text)
                self.populate_listWidgetTables()

    def contextVHeader(self, event):
        print(sys._getframe().f_code.co_name)
        item = self.ui.tableWidget.itemAt(event)
        contextMenu = QMenu(self)
        if item is not None:
            contextMenu.addAction('Delete')
            selectedRowsIndexes = self.ui.tableWidget.selectionModel().selectedRows()
            ids = []
            for indx in selectedRowsIndexes:
                cell = self.ui.tableWidget.item(indx.row(), self.activeTable.columnsNames.index('id'))
                ids.append(cell.text())

        action = contextMenu.exec_(self.ui.tableColsProps.mapToGlobal(event))

        if action is not None:
            if action.text() == 'Delete':
                for row in ids:
                    condition = ('id', row)
                    self.activeTable.deleteRow(condition)
                self.populateTableWidget()

    def createDataBase(self):
        text, okPressed = QInputDialog.getText(self, 'schema name', 'schema name', QLineEdit.Normal, '')
        self.db.create_data_base(text)
        self.populate_listWidgetDataBases()

    def createTable(self):
        print(sys._getframe().f_code.co_name)
        tableName, okPressed = QInputDialog.getText(self, 'table name', 'table name', QLineEdit.Normal, '')
        if tableName == '':
            message = 'Please give a name to table'
            QMessageBox.warning(self, 'Missing Data', message, QMessageBox.Ok)
            return()
        self.activeDataBase.create_new_table(tableName)
        self.populate_listWidgetTables()

    def showColsProps(self):
        print(sys._getframe().f_code.co_name)
        self.ui.tableColsProps.setEditTriggers(QTableWidget.NoEditTriggers)
        if self.ui.tableTab.currentIndex() == 1:
            self.ui.tableColsProps.setRowCount(len(self.activeTable.columnsProperties))
            for row, colName in enumerate(self.activeTable.columnsProperties):
                self.ui.tableColsProps.setItem(row, 0, QTableWidgetItem(colName))
                colType, null, key, default, extra = self.activeTable.columnsProperties[colName]
                # print('{}_{}_{}_{}_{}_{}'.format(colName, colType, null, key, default, extra))
                # print(type(colName), type(colType), type(colName), type(null), type(key), type(default), type(extra))
                if isinstance(default, bytes):
                    default = default.decode('utf-8')
                self.ui.tableColsProps.setItem(row, 1, QTableWidgetItem(colType))

                cell = QTableWidgetItem()
                cell.setFlags(QtCore.Qt.ItemIsUserCheckable | QtCore.Qt.ItemIsEnabled)
                if str(null) == 'NO':
                    cell.setCheckState(QtCore.Qt.Unchecked)
                elif str(null) == 'YES':
                    cell.setCheckState(QtCore.Qt.Checked)
                self.ui.tableColsProps.setItem(row, 2, cell)
                cell = QComboBox()
                keys = ['NONE', 'PRI', 'FOREIGN']
                cell.addItems(keys)
                if key == 'PRI':
                    cell.setCurrentText('PRI')
                else:
                    cell.setCurrentText('NONE')
                cell.currentIndexChanged.connect(lambda i, row=row: self.bufferChangesColsProps(row, 3))

                self.ui.tableColsProps.setCellWidget(row, 3, cell)
                self.ui.tableColsProps.setItem(row, 4, QTableWidgetItem(default))

                cell = QTableWidgetItem()
                cell.setFlags(QtCore.Qt.ItemIsUserCheckable | QtCore.Qt.ItemIsEnabled)
                if 'auto_increment' in extra:
                    cell.setCheckState(QtCore.Qt.Checked)
                else:
                    cell.setCheckState(QtCore.Qt.Unchecked)
                self.ui.tableColsProps.setItem(row, 5, cell)

    def sort(self, logical_index):
        print(sys._getframe().f_code.co_name)
        header = self.ui.tableWidget.horizontalHeader()
        order = Qt.DescendingOrder
        if not header.isSortIndicatorShown():
            header.setSortIndicatorShown(True)
        elif header.sortIndicatorSection() == logical_index:
            order = header.sortIndicatorOrder()
        header.setSortIndicator(logical_index, order)
        self.ui.tableWidget.sortItems(logical_index, order)

    def setFilter(self, logical_index):
        print(sys._getframe().f_code.co_name)
        #toDo fa mai departe
        # print(self.activeTable.columnsNames)
        colName = self.activeTable.columnsNames[logical_index]
        header = self.ui.tableWidget.horizontalHeader()

        geom = QtCore.QRect(header.sectionViewportPosition(logical_index), 0, header.sectionSize(logical_index), header.height())
        item = QLineEdit(header)
        item.setGeometry(geom)
        item.show()
        item.setFocus()
        item.editingFinished.connect(lambda: (self.applyFilter(colName, item.text()),
                                              item.clear(),
                                              item.hide(),
                                              item.deleteLater()))

    def applyFilter(self, colName, filter):
        print(sys._getframe().f_code.co_name)
        if filter == '':
            return

        if self.defaultFilter:
            self.ui.lineEditFilterList.clear()
            self.defaultFilter = False

        filterText = self.ui.lineEditFilterList.text()
        if not filterText:
            if isinstance(filter, str):
                filterText += '{}="{}"'.format(colName, filter)
            elif isinstance(filter, tuple):
                filterText += '{} < {} < {}'.format(filter[0], colName, filter[1])
            elif isinstance(filter, list):
                filterText += '{} in {}"'.format(str(filter), colName)
        else:
            if isinstance(filter, str):
                filterText += '; {}="{}"'.format(colName, filter)
            elif isinstance(filter, tuple):
                filterText += '; {} < {} < {}'.format(filter[0], colName, filter[1])
            elif isinstance(filter, list):
                filterText += '; {} in {}"'.format(str(filter), colName)

        self.ui.lineEditFilterList.setText(filterText)

        tup = (colName, filter)
        self.filterList.append(tup)
        self.data = self.activeTable.filterRows(self.filterList)
        self.data = np.atleast_2d(self.data)
        if self.data.shape == (1, 0):
            self.data = np.empty((0, len(self.activeTable.columnsNames)))

        self.populateTable()

    def importCSV(self):
        print(sys._getframe().f_code.co_name)
        inpFile, _ = QFileDialog.getOpenFileName(None, 'Select .csv file', '', 'CSV files (*.csv)')
        with open(inpFile, 'r', encoding='unicode_escape', newline='') as csvfile:
            linereader = csv.reader(csvfile, delimiter=';', quotechar='|')
            for i, row in enumerate(linereader):
                if i == 0:
                    tableHead = row

        for col in tableHead:
            if col == 'id':
                continue
            if col not in self.activeTable.columnsNames:
                message = 'column {} missing in SQL table'.format(col)
                QMessageBox.warning(self, 'Inconsistent Data', message, QMessageBox.Ok)
                return ()

        self.activeTable.importCSV(inpFile)
        self.populateTableWidget()

    def exportCSV(self):
        inpFile, _ = QFileDialog.getSaveFileName(None, 'Select .csv file', '', 'CSV files (*.csv)')

        tableHead = []
        for colNo in range(self.ui.tableWidget.columnCount()):
            colName = self.ui.tableWidget.horizontalHeaderItem(colNo).text()
            tableHead.append(colName)

        array = self.readTable()
        with open(inpFile, mode='w', newline='') as csvFile:
            fileWriter = csv.writer(csvFile, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
            fileWriter.writerow(tableHead)
            for i in array:
                fileWriter.writerow(i)


class CreateTable(QDialog):
    def __init__(self, tableName):
        super(CreateTable, self).__init__()
        Ui_createTableWindow, QtBaseClass = uic.loadUiType('createTable.ui')
        self.gui = Ui_createTableWindow()
        self.gui.setupUi(self)
        self.gui.toolButtonAddRow.clicked.connect(self.addRow)
        self.tableName = tableName

    def addRow(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        row = self.gui.tableWidgetCreateTable.rowCount()
        self.gui.tableWidgetCreateTable.insertRow(row)

        for col in range(self.gui.tableWidgetCreateTable.columnCount()):
            headerName = self.gui.tableWidgetCreateTable.horizontalHeaderItem(col).text()
            if headerName == 'Type':
                cell = QComboBox()
                self.dataTypes = ['INT', 'FLOAT', 'VARCHAR', 'TEXT', 'DATE']
                cell.addItems(self.dataTypes)
                self.gui.tableWidgetCreateTable.setCellWidget(row, col, cell)
                # cell.currentIndexChanged.connect(self.setDefaultDataTypeLength)

    # def setDefaultDataTypeLength(self, item):
    #     print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    #     print(item, type(item))
    #     print(self.dataTypes[item])
    #     # print(row, col)

    def tableQuery(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        print(sys._getframe().f_code.co_name)
        print('caller: ', sys._getframe().f_back.f_code.co_name)
        #todo cand se modifica col type in varchar automat are nevoie de length
        rows = self.gui.tableWidgetCreateTable.rowCount()
        cols = self.gui.tableWidgetCreateTable.columnCount()
        tableHead = []
        for col in range(self.gui.tableWidgetCreateTable.columnCount()):
            headerName = self.gui.tableWidgetCreateTable.horizontalHeaderItem(col).text()
            tableHead.append(headerName)

        query = "CREATE TABLE IF NOT EXISTS {} ( ".format(self.tableName)
        extra = ''
        for row in range(rows):
            for column in range(cols):
                cell = self.gui.tableWidgetCreateTable.item(row, column)
                headerName = self.gui.tableWidgetCreateTable.horizontalHeaderItem(column).text()
                if headerName == 'Name':
                    if cell is None:
                        message = 'Please give a name to column'
                        QMessageBox.warning(self, 'Missing Data', message, QMessageBox.Ok)
                        return ()
                    name = cell.text()
                    rowQuery = '{} '.format(name)
                elif headerName == 'Type':
                    widget = self.gui.tableWidgetCreateTable.cellWidget(row, column)
                    type = widget.currentText()
                    rowQuery += '{}'.format(type)
                elif headerName == 'Length':
                    if cell is not None:
                        lengthVal = '({})'.format(cell.text())
                        rowQuery += lengthVal
                elif headerName == 'Null':
                    if cell is not None and cell.checkState() == QtCore.Qt.Unchecked:
                        rowQuery += ' NOT NULL '
                elif headerName == 'Default':
                    if cell is None:
                        if 'NOT NULL' not in rowQuery:
                            rowQuery += ' DEFAULT NULL '
                    else:
                        rowQuery += ' {} '.format(cell.text())
                elif headerName == 'AutoIncrement':
                    if cell and cell.checkState() == QtCore.Qt.Checked:
                        # extra += '\n ALTER TABLE {} MODIFY {} {}{} AUTO_INCREMENT;'.format(self.tableName, name, type, lengthVal)
                        rowQuery += ' AUTO_INCREMENT '
                elif headerName == 'Key':
                    widget = self.gui.tableWidgetCreateTable.cellWidget(row, column)
                    keyType = widget.currentText()
                    if keyType == 'None':
                        continue
                    elif keyType == 'PRIMARY KEY':
                        extra += '\n ALTER TABLE {} ADD PRIMARY KEY ({});'.format(self.tableName, name)
                    elif keyType == 'FOREIGN KEY':
                        refTableindx = tableHead.index('Ref_Table')
                        refColindx = tableHead.index('Ref_Column')
                        refTab = self.gui.tableWidgetCreateTable.item(row, refTableindx).text()
                        refCol = self.gui.tableWidgetCreateTable.item(row, refColindx).text()
                        contraint = '´{}_ibfk´'.format(refTab)
                        extra += '\n ALTER TABLE {} ' \
                                 'ADD CONSTRAINT {} FOREIGN KEY ({}) ' \
                                 'REFERENCES {} ({}) ON DELETE CASCADE ON UPDATE CASCADE;'.format(self.tableName, contraint, name, refTab, refCol)

                    # rowQuery += ', \n'
            rowQuery += ', '
            query += rowQuery

        # print('_{}_'.format(query))
        query = query[:-2]
        query += ' );'

        completeQuery = query + extra
        print(50*'--')
        print(completeQuery)
        print(50 * '--')

        return completeQuery

    def colQuery(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        rows = self.gui.tableWidgetCreateTable.rowCount()
        cols = self.gui.tableWidgetCreateTable.columnCount()

        newRows = []
        for row in range(rows):
            for column in range(cols):
                cell = self.gui.tableWidgetCreateTable.item(row, column)
                headerName = self.gui.tableWidgetCreateTable.horizontalHeaderItem(column).text()
                if headerName == 'Name':
                    if cell is None:
                        message = 'Please give a name to column'
                        QMessageBox.warning(self, 'Missing Data', message, QMessageBox.Ok)
                        return
                    name = cell.text()
                elif headerName == 'Type':
                    widget = self.gui.tableWidgetCreateTable.cellWidget(row, column)
                    typeCol = widget.currentText()
                elif headerName == 'Length':
                    if cell is not None:
                        lengthVal = cell.text()
                    elif cell is None and typeCol == 'VARCHAR':
                        message = 'Please give a length to column'
                        QMessageBox.warning(self, 'Missing Data', message, QMessageBox.Ok)
                        return
                    else:
                        lengthVal = None
            tup = (name, typeCol, lengthVal)
            newRows.append(tup)
        return newRows

    @staticmethod
    def createTableQuery(tableName):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        dialog = CreateTable(tableName)
        result = dialog.exec_()
        query = dialog.tableQuery()
        if result == QDialog.Accepted:
            return query
        else:
            return None

    @staticmethod
    def createColumnQuery(tableName):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        dialog = CreateTable(tableName)
        result = dialog.exec_()
        query = dialog.colQuery()
        if result == QDialog.Accepted:
            return query
        else:
            return None


class InsertNewRow(QDialog):
    def __init__(self, cols):
        super(InsertNewRow, self).__init__()
        self.cols = cols
        path2src, pyFileName = os.path.split(__file__)
        uiFileName = 'addNewRow.ui'
        path2GUI = os.path.join(path2src, 'GUI', uiFileName)
        Ui_MainWindow, QtBaseClass = uic.loadUiType(path2GUI)
        self.gui = Ui_MainWindow()
        self.gui.setupUi(self)
        self.fillInTable()

    def fillInTable(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.gui.tableWidget.setRowCount(len(self.cols))
        for row, colName in enumerate(self.cols):
            cell = QLineEdit()
            cell.setText(str(colName))
            cell.setEnabled(False)
            self.gui.tableWidget.setCellWidget(row, 0, cell)

            colType, null, key, default, extra = self.cols[colName][:5]
            cell = QLineEdit()
            cell.setText(str(colType))
            cell.setEnabled(False)
            self.gui.tableWidget.setCellWidget(row, 1, cell)
            # self.gui.tableWidget.setItem(row, 1, QTableWidgetItem(colType))
            cell = QLineEdit()
            cell.setText(str(null))
            cell.setEnabled(False)
            self.gui.tableWidget.setCellWidget(row, 2, cell)
            # self.gui.tableWidget.setItem(row, 2, QTableWidgetItem(null))
            cell = QLineEdit()
            cell.setText(str(key))
            cell.setEnabled(False)
            self.gui.tableWidget.setCellWidget(row, 3, cell)
            # self.gui.tableWidget.setItem(row, 3, QTableWidgetItem(key))
            cell = QLineEdit()
            cell.setText(str(default))
            cell.setEnabled(False)
            self.gui.tableWidget.setCellWidget(row, 4, cell)
            # self.gui.tableWidget.setItem(row, 4, QTableWidgetItem(default))

            cellAutoIncr = QTableWidgetItem()
            cellAutoIncr.setFlags(QtCore.Qt.ItemIsEnabled)#QtCore.Qt.ItemIsUserCheckable |

            if 'auto_increment' in extra:
                autoIncremVal = self.cols[colName][-1]
                cell = QLineEdit()
                cell.setText(str(autoIncremVal))
                cell.setEnabled(False)
                cellAutoIncr.setCheckState(QtCore.Qt.Checked)
                self.gui.tableWidget.setCellWidget(row, 6, cell)
            else:
                cellAutoIncr.setCheckState(QtCore.Qt.Unchecked)
                self.gui.tableWidget.setItem(row, 6, QTableWidgetItem(extra))
            self.gui.tableWidget.setItem(row, 5, cellAutoIncr)

            if 'timestamp' in colType:
                cell = QDateEdit()
                cell.setDisplayFormat('yyyy-MM-dd hh:mm:ss')
                cell.setDateTime(QDateTime(datetime.now().year,
                                           datetime.now().month,
                                           datetime.now().day,
                                           datetime.now().hour,
                                           datetime.now().minute,
                                           datetime.now().second))
                cell.setEnabled(False)
                self.gui.tableWidget.setCellWidget(row, 6, cell)

            if 'date' in colType:
                cell = QDateEdit()
                cell.setDisplayFormat('yyyy-MM-dd')
                cell.setDate(QDate(datetime.now().year,
                                           datetime.now().month,
                                           datetime.now().day))
                # cell.setEnabled(False)
                self.gui.tableWidget.setCellWidget(row, 6, cell)
            elif ('smallint' in colType or 'tinyint' in colType) and 'active' in colName:
                cell = QTableWidgetItem()
                cell.setFlags(QtCore.Qt.ItemIsUserCheckable | QtCore.Qt.ItemIsEnabled)
                cell.setCheckState(QtCore.Qt.Checked)
                self.gui.tableWidget.setCellWidget(row, 6, cell)

    def readTable(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        rows = self.gui.tableWidget.rowCount()
        cols = self.gui.tableWidget.columnCount()

        columns = []
        values = []

        for row in range(rows):
            for column in range(cols):
                cell = self.gui.tableWidget.item(row, column)
                headerName = self.gui.tableWidget.horizontalHeaderItem(column).text()
                if headerName == 'ColName':
                    # colName = cell.text()
                    widget = self.gui.tableWidget.cellWidget(row, column)
                    colName = widget.text()
                if headerName == 'Value':
                    widget = self.gui.tableWidget.cellWidget(row, column)
                    if cell is not None:
                        if cell.text():
                            colValue = cell.text()
                            print(colValue)
                        elif cell.checkState() == QtCore.Qt.Checked:
                            colValue = 1
                        elif cell.checkState() == QtCore.Qt.Unchecked:
                            colValue = 0
                        columns.append(colName)
                        values.append(colValue)
                    elif widget:
                        if isinstance(widget, QDateTimeEdit):
                            colValue = widget.dateTime()
                            colValue = colValue.toPyDateTime()
                            columns.append(colName)
                            values.append(str(colValue))
        return tuple(columns), tuple(values)

    @staticmethod
    def getNewRowValues(cols):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        dialog = InsertNewRow(cols)
        result = dialog.exec_()
        cols, values = dialog.readTable()
        if result == QDialog.Accepted:
            return cols, values
        else:
            return None


if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = MyApp()
    window.show()
    sys.exit(app.exec_())



import numpy as np
from PyQt5.QtWidgets import *
from PyQt5 import uic
from PyQt5 import QtCore
from PyQt5.QtCore import Qt, QDateTime, QTime
import sip
import pathlib
import sys
from datetime import datetime
from mysqlquerys import connect
from mysqlquerys import postgresql_rm
from mysqlquerys import mysql_rm
import inspect
np.set_printoptions(linewidth=600)


class CheckRequiredFiles:
    def __init__(self):
        path2GUI = pathlib.Path(__file__)
        self.path2GUI = path2GUI.resolve(path2GUI).parent / 'gui'
        self.gui = self.path2GUI / 'gui.ui'
        self.postgresql_gui = self.path2GUI / 'postgresql_gui.ui'
        self.mysql_gui = self.path2GUI / 'postgresql_gui.ui'
        self.filterWindow = self.path2GUI / 'filterWindow.ui'
        self.inputsWindow = self.path2GUI / 'inputs_sql.ui'


class guiSqlRM(QMainWindow, CheckRequiredFiles):
    def __init__(self, connection):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        super(guiSqlRM, self).__init__()
        Ui_MysqlWindow, QtBaseClass = uic.loadUiType(self.mysql_gui)
        self.ui = Ui_MysqlWindow()
        self.ui.setupUi(self)
        self.credentials = connection.credentials
        self.db_type = connection.db_type
        if self.db_type == 'mysql':
            self.sql_rm = mysql_rm
            self.ui.TBox_db_sch_tab.setItemEnabled(1, False)
        elif self.db_type == 'postgresql':
            self.sql_rm = postgresql_rm

        self.ui.GB_db_defs.setVisible(False)
        self.ui.GB_schema_defs.setVisible(False)
        self.ui.LW_schemas_available_in_db.setVisible(False)
        self.ui.GB_table_defs.setVisible(False)

        self.GB_db_defs = self.ui.GB_db_defs
        self.GB_db_not_callable = self.ui.GB_db_not_callable
        self.GB_db_callable_without_args = self.ui.GB_db_callable_without_args
        self.GB_db_callable_with_args = self.ui.GB_db_callable_with_args
        self.GB_schema_not_callable = self.ui.GB_schema_not_callable
        self.GB_schema_callable_without_args = self.ui.GB_schema_callable_without_args
        self.GB_schema_callable_with_args = self.ui.GB_schema_callable_with_args
        self.GB_table_defs = self.ui.GB_table_defs
        self.GB_table_not_callable = self.ui.GB_table_not_callable
        self.GB_table_callable_without_args = self.ui.GB_table_callable_without_args
        self.GB_table_callable_with_args = self.ui.GB_table_callable_with_args
        self.ui.TBox_db_sch_tab.setCurrentIndex(0)

        self.ui.CheckB_db_methods.toggled.connect(self.populate_class_methods)
        self.ui.CheckB_schema_methods.toggled.connect(self.populate_class_methods)
        self.ui.CheckB_table_methods.toggled.connect(self.populate_class_methods)

        self.db = self.sql_rm.DataBase(self.credentials)
        title = '{} {}'.format(self.credentials['database'], self.db.dataBaseVersion)
        self.setWindowTitle(title)

        self.ui.CheckB_hide_schemas_list.toggled.connect(lambda: self.populate_list(self.db.allAvailableTablesInDatabase, self.ui.LW_schemas_available_in_db))
        self.ui.CheckB_hide_tables_list.toggled.connect(lambda: self.populate_list(self.schema.allAvailableTablesInSchema, self.ui.LW_tables_available))
        self.ui.CheckB_hide_table.toggled.connect(lambda: self.hide_table_widget(self.ui.TW_table_data))

        if self.db_type == 'mysql':
            self.populate_list(self.db.allAvailableTablesInDatabase, self.ui.LW_schemas_available_in_db)
            self.ui.LW_schemas_available_in_db.itemDoubleClicked.connect(self.connect_to_table)
        elif self.db_type == 'postgresql':
            self.populate_list(self.db.allAvailableSchemas, self.ui.LW_schemas_available_in_db)
            self.ui.LW_schemas_available_in_db.itemDoubleClicked.connect(self.connect_to_schema)
            self.ui.LW_tables_available.itemDoubleClicked.connect(self.connect_to_table)

        self.ui.TW_table_data.horizontalHeader().sectionClicked.connect(self.sort)
        self.ui.TW_table_data.horizontalHeader().sectionDoubleClicked.connect(self.setFilter)
        self.ui.PB_default_filter.clicked.connect(self.set_default_filter_filter)
        self.ui.PB_show_full_table.clicked.connect(self.show_full_table)

    def populate_list(self, list_items, list_widget):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if isinstance(self.sender(), QCheckBox):
            if self.sender().isChecked():
                list_widget.setVisible(False)
                list_widget.clear()
                for i, schema in enumerate(list_items):
                    list_widget.insertItem(i, schema)
            else:
                list_widget.setVisible(True)
        elif isinstance(self.sender(), QPushButton):
            list_widget.setVisible(True)
            list_widget.clear()
            for i, schema in enumerate(list_items):
                list_widget.insertItem(i, schema)

    def hide_table_widget(self, table_widget):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if isinstance(self.sender(), QCheckBox):
            if self.sender().isChecked():
                table_widget.setVisible(False)
            else:
                table_widget.setVisible(True)

    def populate_class_methods(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if 'DataBase' in self.sender().text():
            module_class = self.sql_rm.DataBase
            module_instance = self.db
            module_class_name = 'self.sql_rm.DataBase'
            group_box = self.GB_db_defs
            GB_not_callable = self.GB_db_not_callable
            GB_callable_without_args = self.GB_db_callable_without_args
            GB_callable_with_args = self.GB_db_callable_with_args
            definitions = [x for x in dir(self.sql_rm.DataBase) if not x.startswith('__')]
        elif 'Schema' in self.sender().text():
            module_class = self.sql_rm.Schema
            module_instance = self.schema
            module_class_name = 'self.sql_rm.Schema'
            group_box = self.GB_schema_defs
            GB_not_callable = self.GB_schema_not_callable
            GB_callable_without_args = self.GB_schema_callable_without_args
            GB_callable_with_args = self.GB_schema_callable_with_args
            definitions_DB = [x for x in dir(self.sql_rm.DataBase) if not x.startswith('__')]
            definitions = [x for x in dir(self.sql_rm.Schema) if not x.startswith('__')]
            for item in definitions:
                if item in definitions_DB:
                    definitions.remove(item)
        elif 'Table' in self.sender().text():
            module_class = self.sql_rm.Table
            module_instance = self.table
            module_class_name = 'self.sql_rm.Table'
            group_box = self.GB_table_defs
            GB_not_callable = self.GB_table_not_callable
            GB_callable_without_args = self.GB_table_callable_without_args
            GB_callable_with_args = self.GB_table_callable_with_args
            definitions_DB = [x for x in dir(self.sql_rm.DataBase) if not x.startswith('__')]
            definitions = [x for x in dir(self.sql_rm.Table) if not x.startswith('__')]
            new_def = []
            for item in definitions:
                if item in definitions_DB:
                    continue
                else:
                    new_def.append(item)
            definitions = new_def

        if isinstance(self.sender(), QCheckBox):
            if self.sender().isChecked():
                group_box.setVisible(True)
                vbox_not_callable = QGridLayout()
                vbox_callable_without_args = QGridLayout()
                vbox_callable_with_args = QGridLayout()
                for i, meth_name in enumerate(definitions):
                    radio = QRadioButton(meth_name)
                    radio.col = 0
                    radio.row = i
                    radio.module_class = module_class
                    radio.module_instance = module_instance
                    radio.meth_name = meth_name
                    meth = getattr(module_class, meth_name)
                    if callable(meth):
                        print('meth', meth_name)
                        docs = None
                        # if hasattr(meth, '__doc__'):
                        #     docs = getattr(meth, '__doc__')
                        varname = 'def_args'
                        loc = locals()
                        exec('{}=inspect.getfullargspec({}.{})'.format(varname, module_class_name, meth_name), globals(), loc)
                        meth_args = (loc[varname]).args
                        coloana = 1
                        if len(meth_args) == 1 and meth_args[0] == 'self':
                            radio.toggled.connect(self.start_noncallable_method)
                            vbox_callable_without_args.addWidget(radio, i, radio.col)
                        else:
                            vbox_callable_with_args.addWidget(radio, i, radio.col)
                            tool_button = QPushButton('inputs')
                            tool_button.meth_name = meth_name
                            tool_button.clicked.connect(self.get_method_arguments)
                            vbox_callable_with_args.addWidget(tool_button, i, 2)
                            push_button = QPushButton('Fire')
                            push_button.row = len(definitions) + 1
                            push_button.col = 2
                            push_button.clicked.connect(self.start_callable_method)
                            vbox_callable_with_args.addWidget(push_button, push_button.row, push_button.col)
                    else:
                        radio.toggled.connect(self.start_noncallable_method)
                        vbox_not_callable.addWidget(radio, i, radio.col)
                GB_not_callable.setLayout(vbox_not_callable)
                GB_callable_without_args.setLayout(vbox_callable_without_args)
                GB_callable_with_args.setLayout(vbox_callable_with_args)
            else:
                group_box.setVisible(False)

    def get_method_arguments(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if self.ui.TBox_db_sch_tab.currentIndex() == 0:
            layout_grupa = self.GB_callable_with_args.layout()
            module_class_name = 'self.sql_rm.DataBase'
            module_instance = self.db
        elif self.ui.TBox_db_sch_tab.currentIndex() == 1:
            layout_grupa = self.GB_schema_callable_with_args.layout()
            module_class_name = 'self.sql_rm.DataBase'
            module_instance = self.db
        elif self.ui.TBox_db_sch_tab.currentIndex() == 2:
            layout_grupa = self.GB_table_callable_with_args.layout()
            module_class_name = 'self.sql_rm.Table'
            module_instance = self.table
        args_dict = {}
        meth = None
        for x in range(layout_grupa.count()):
            item = layout_grupa.itemAt(x)
            widget = item.widget()
            if isinstance(widget, QRadioButton):
                if widget.isChecked():
                    meth = widget.text()
                    print(meth)
                    varname = 'def_args'
                    loc = locals()
                    exec('{}=inspect.getfullargspec({}.{})'.format(varname, module_class_name, meth), globals(), loc)
                    meth_args = (loc[varname]).args
                    print('\t', meth_args)

                    docs = None
                    if hasattr(meth, '__doc__'):
                        docs = getattr(meth, '__doc__')
                    for co, arg in enumerate(meth_args):
                        if arg == 'self':
                            continue

                        if docs:
                            for line in docs.split('\n'):
                                line = line.strip()
                                print(line)
                                if ':param' in line and arg in line:
                                    if 'QFileDialog.getOpenFileNames' in line:
                                        print('BINGO')

    def connect_to_schema(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        schema_name = self.ui.LW_schemas_available_in_db.currentItem().text()
        self.schema = self.sql_rm.Schema(self.credentials, schema_name)
        self.schema.set_schema_as_default()
        self.ui.TBox_db_sch_tab.setCurrentIndex(1)
        list_items = self.schema.allAvailableTablesInSchema
        list_widget = self.ui.LW_tables_available
        list_widget.setVisible(True)
        list_widget.clear()
        for i, schema in enumerate(list_items):
            list_widget.insertItem(i, schema)

    def connect_to_table(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.filterList = []
        if self.db_type == 'mysql':
            table_name = self.ui.LW_schemas_available_in_db.currentItem().text()
            self.table = self.sql_rm.Table(self.credentials, table_name)
        elif self.db_type == 'postgresql':
            schema_name = self.ui.LW_schemas_available_in_db.currentItem().text()
            table_name = self.ui.LW_tables_available.currentItem().text()
            self.table = self.sql_rm.Table(self.credentials, schema_name, table_name)

        self.ui.LE_tot_no_of_rows.setText(str(self.table.noOfRows))
        self.ui.TBox_db_sch_tab.setCurrentIndex(2)
        table_data = self.table.returnLastRecords('id', 100)
        table_data = np.atleast_2d(table_data)
        self.ui.lineEditFilterList.setText("last 100 id's")
        self.populate_table(table_data)

    def populate_table(self, table_data):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.ui.TW_table_data.clear()
        self.ui.TW_table_data.setRowCount(table_data.shape[0])
        self.ui.TW_table_data.setColumnCount(table_data.shape[1])
        tableHead = self.table.columnsNames
        self.ui.TW_table_data.setHorizontalHeaderLabels(tableHead)
        for col in range(table_data.shape[1]):
            for row in range(table_data.shape[0]):
                if isinstance(table_data[row, col], int):
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, table_data[row, col])
                else:
                    item = QTableWidgetItem(str(table_data[row, col]))
                self.ui.TW_table_data.setItem(row, col, item)
        self.ui.LE_showing.setText(str(table_data.shape[0]))

    def deleteItemsOfLayout(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        layout_grupa = self.GB_db_defs.layout()
        if layout_grupa is not None:
            while layout_grupa.count():
                item = layout_grupa.takeAt(0)
                widget = item.widget()
                if widget is not None:
                    widget.setParent(None)
                else:
                    self.deleteItemsOfLayout(item.layout())
            sip.delete(layout_grupa)

    def start_noncallable_method(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if self.sender().isChecked():
            method = self.sender().text()
            if not callable(getattr(self.sender().module_class, method)):
                text = getattr(self.sender().module_instance, self.sender().text())
                self.ui.TE_result.setPlainText(str(text))
            else:
                text = getattr(self.sender().module_instance, self.sender().text())()
                self.ui.TE_result.setPlainText(str(text))

    def start_callable_method(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if self.ui.TBox_db_sch_tab.currentIndex() == 0:
            layout_grupa = self.GB_db_defs.layout()
            module_class_name = 'self.sql_rm.DataBase'
            module_instance = self.db
        elif self.ui.TBox_db_sch_tab.currentIndex() == 1:
            layout_grupa = self.GB_schema_defs.layout()
            module_class_name = 'self.sql_rm.Schema'
            module_instance = self.schema
        elif self.ui.TBox_db_sch_tab.currentIndex() == 2:
            layout_grupa = self.GB_table_defs.layout()
            module_class_name = 'self.sql_rm.Table'
            module_instance = self.table

        args_dict = {}
        meth = None
        for x in range(layout_grupa.count()):
            item = layout_grupa.itemAt(x)
            widget = item.widget()
            if isinstance(widget, QRadioButton):
                if widget.isChecked():
                    meth = widget.text()
                    print(meth)
                    varname = 'def_args'
                    loc = locals()
                    exec('{}=inspect.getfullargspec({}.{})'.format(varname, module_class_name, meth), globals(), loc)
                    meth_args = (loc[varname]).args
                    for arg in meth_args:
                        if arg == 'self':
                            continue
                        for x in range(layout_grupa.count()):
                            item = layout_grupa.itemAt(x)
                            widget = item.widget()
                            if isinstance(widget, QLineEdit):
                                if widget.arument == arg:
                                    args_dict[arg] = widget.text()
                                    break
                            elif isinstance(widget, QListWidget):
                                if widget.arument == arg:
                                    args_dict = []
                                    for i in range(widget.count()):
                                        args_dict.append(widget.item(i).text())
                                        break
        if not meth:
            message = "Please select the a method"
            QMessageBox.warning(self, 'Fehlende Daten', message, QMessageBox.Ok)
            return

        text = getattr(module_instance, meth)(**args_dict)
        self.ui.TE_result.setPlainText(str(text))

    def sort(self, logical_index):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        header = self.ui.TW_table_data.horizontalHeader()
        order = Qt.DescendingOrder
        if not header.isSortIndicatorShown():
            header.setSortIndicatorShown(True)
        elif header.sortIndicatorSection() == logical_index:
            order = header.sortIndicatorOrder()
        header.setSortIndicator(logical_index, order)
        self.ui.TW_table_data.sortItems(logical_index, order)

    def setFilter(self, logical_index):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        colName = self.table.columnsNames[logical_index]

        if self.table.columnsProperties[colName] == 'datetime' or \
                self.table.columnsProperties[colName] == 'timestamp' or \
                self.table.columnsProperties[colName] == 'timestamp without time zone':
            filt = FilterWindow.getDateTimeInterval(colName)
            if not filt:
                return
            if isinstance(filt, tuple):
                minDate, maxDate = filt
                minDate = minDate.toPyDateTime()
                maxDate = maxDate.toPyDateTime()
                filterVals = (str(minDate), str(maxDate))
            elif isinstance(filt, QDateTime):
                filterVals = filt.toPyDateTime()
            self.applyFilter(colName, filterVals)
        elif self.table.columnsProperties[colName] == 'time' or \
                self.table.columnsProperties[colName] == 'time without time zone':
            filt = FilterWindow.getTimeInterval(colName)
            if not filt:
                return
            if isinstance(filt, tuple):
                minTime, maxTime = filt
                minTime = minTime.toPyTime()
                maxTime = maxTime.toPyTime()
                filterVals = (str(minTime), str(maxTime))
            elif isinstance(filt, QTime):
                filterVals = filt.toPyTime()
            self.applyFilter(colName, filterVals)
        elif self.table.columnsProperties[colName] == 'int' or \
                self.table.columnsProperties[colName] == 'integer' or \
                self.table.columnsProperties[colName] == 'mediumint':
            filt = FilterWindow.getIntInterval(colName)
            if not filt:
                return
            if isinstance(filt, tuple):
                minTime, maxTime = filt
                filterVals = (str(minTime), str(maxTime))
            elif isinstance(filt, str):
                filterVals = filt
            self.applyFilter(colName, filterVals)
        else:
            header = self.ui.TW_table_data.horizontalHeader()
            geom = QtCore.QRect(header.sectionViewportPosition(logical_index), 0, header.sectionSize(logical_index),
                                header.height())
            item = QLineEdit(header)
            item.setGeometry(geom)
            item.show()
            item.setFocus()
            item.editingFinished.connect(lambda: (self.applyFilter(colName, item.text()), item.clear(),
                                                  item.hide(), item.deleteLater()))

    def applyFilter(self, colName, filter):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if filter == "":
            return

        filterText = self.ui.lineEditFilterList.text()
        if filterText == "last 100 id's":
            filterText = ""
            self.ui.lineEditFilterList.setText(filterText)
        if not filterText:
            if isinstance(filter, str):
                filterText += '{}="{}"'.format(colName, filter)
            elif isinstance(filter, tuple):
                filterText += '{} < {} < {}'.format(filter[0], colName, filter[1])
            elif isinstance(filter, list):
                filterText += '{} in {}'.format(str(filter), colName)
        else:
            if isinstance(filter, str):
                filterText += '; {}="{}"'.format(colName, filter)
            elif isinstance(filter, tuple):
                filterText += '; {} < {} < {}'.format(filter[0], colName, filter[1])
            elif isinstance(filter, list):
                filterText += '{} in {}'.format(str(filter), colName)

        self.ui.lineEditFilterList.setText(filterText)
        tup = (colName, filter)
        self.filterList.append(tup)

        table_data = self.table.filterRows(self.filterList)

        if table_data:
            table_data = np.atleast_2d(table_data)
        else:
            table_data = np.zeros((0, len(self.tableHead)), dtype=str)

        self.populate_table(table_data)

    def set_default_filter_filter(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        table_data = self.table.returnLastRecords('id', 100)
        table_data = np.atleast_2d(table_data)
        self.populate_table(table_data)
        self.ui.lineEditFilterList.setText("last 100 id's")

    def show_full_table(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        table_data = self.table.returnAllRecordsFromTable()
        table_data = np.atleast_2d(table_data)
        self.populate_table(table_data)
        self.ui.lineEditFilterList.setText("")


class FilterWindow(QDialog, CheckRequiredFiles):
    def __init__(self, colType, colName):
        super(FilterWindow, self).__init__()
        self.colName = colName
        Ui_MainWindow, QtBaseClass = uic.loadUiType(self.filterWindow)
        self.gui = Ui_MainWindow()
        self.gui.setupUi(self)
        self.gui.GBDateTime.setVisible(False)
        self.gui.GBLineEdit.setVisible(False)
        self.gui.GBTime.setVisible(False)
        self.gui.checkBoxDateTime.stateChanged.connect(self.openDateTimeInterval)
        self.gui.checkBoxTime.stateChanged.connect(self.openTimeInterval)
        self.gui.checkBoxMaxVal.stateChanged.connect(self.openValInterval)
        if colType == 'dateTime':
            self.gui.GBDateTime.setVisible(True)
            self.gui.dateTimeEdit_MaxVal.setEnabled(False)
            self.gui.label_dateTime.setText('= {}'.format(self.colName))
            self.gui.dateTimeEdit_MinVal.setDateTime(QDateTime(datetime.now().year,
                                                               datetime.now().month,
                                                               1, 0, 0, 0))
            self.gui.dateTimeEdit_MaxVal.setDateTime(QDateTime(datetime.now().year,
                                                               datetime.now().month,
                                                               datetime.now().day,
                                                               datetime.now().hour,
                                                               datetime.now().minute,
                                                               datetime.now().second))
        elif colType == 'time':
            self.gui.GBTime.setVisible(True)
            self.gui.timeEditMax.setEnabled(False)
        elif colType == 'int':
            self.gui.GBLineEdit.setVisible(True)
            self.gui.lineEditMax.setEnabled(False)

        self.rejected.connect(self.byebye)

    def openDateTimeInterval(self):
        if self.gui.checkBoxDateTime.isChecked():
            self.gui.dateTimeEdit_MaxVal.setEnabled(True)
            self.gui.label_dateTime.setText('< {} <'.format(self.colName))
        else:
            self.gui.dateTimeEdit_MaxVal.setEnabled(False)
            self.gui.label_dateTime.setText('= {}'.format(self.colName))

    def openTimeInterval(self):
        if self.gui.checkBoxTime.isChecked():
            self.gui.timeEditMax.setEnabled(True)
            self.gui.label_Time.setText('< {} <'.format(self.colName))
        else:
            self.gui.timeEditMax.setEnabled(False)
            self.gui.label_Time.setText('= {}'.format(self.colName))

    def openValInterval(self):
        if self.gui.checkBoxMaxVal.isChecked():
            self.gui.lineEditMax.setEnabled(True)
            self.gui.label_val.setText('< {} <'.format(self.colName))
        else:
            self.gui.lineEditMax.setEnabled(False)
            self.gui.label_val.setText('= {}'.format(self.colName))

    def byebye(self):
        self.close()

    def dateTimeInterval(self):
        if self.gui.checkBoxDateTime.isChecked():
            tup = (self.gui.dateTimeEdit_MinVal.dateTime(), self.gui.dateTimeEdit_MaxVal.dateTime())
            return tup
        else:
            return self.gui.dateTimeEdit_MinVal.dateTime()

    def timeInterval(self):
        if self.gui.checkBoxTime.isChecked():
            tup = (self.gui.timeEditMin.time(), self.gui.timeEditMax.time())
            return tup
        else:
            return self.gui.timeEditMin.time()

    def intInterval(self):
        if self.gui.checkBoxMaxVal.isChecked():
            tup = (self.gui.lineEditMin.text(), self.gui.lineEditMax.text())
            return tup
        else:
            return self.gui.lineEditMin.text()

    @staticmethod
    def getDateTimeInterval(colName):
        dialog = FilterWindow('dateTime', colName)
        result = dialog.exec_()
        filt = dialog.dateTimeInterval()
        if result == QDialog.Accepted:
            return filt
        else:
            return None

    @staticmethod
    def getTimeInterval(colName):
        dialog = FilterWindow('time', colName)
        result = dialog.exec_()
        filt = dialog.timeInterval()
        if result == QDialog.Accepted:
            return filt
        else:
            return None

    @staticmethod
    def getIntInterval(colName):
        dialog = FilterWindow('int', colName)
        result = dialog.exec_()
        filt = dialog.intInterval()
        if result == QDialog.Accepted:
            return filt
        else:
            return None


class InputsSQL(QDialog, CheckRequiredFiles):
    def __init__(self, colType, colName):
        super(InputsSQL, self).__init__()
        Ui_MainWindow, QtBaseClass = uic.loadUiType(self.inputsWindow)
        self.gui = Ui_MainWindow()
        self.gui.setupUi(self)


class window1(QMainWindow, CheckRequiredFiles):
    def __init__(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        super(window1, self).__init__()
        Ui_MainWindow, QtBaseClass = uic.loadUiType(self.gui)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.conf = None
        self.GB_dbs_available = self.ui.GB_dbs_available
        self.ui.PB_connect_to_db.setVisible(False)
        self.ui.GB_dbs_available.setVisible(False)
        self.ui.TB_load_config_ini.clicked.connect(lambda: (self.load_config_ini(), self.read_config_ini()))
        self.ui.PB_connect_to_db.clicked.connect(self.connect_to_db)

    def load_config_ini(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        inpFile, _ = QFileDialog.getOpenFileName(None, "", "", '*.ini')
        if not inpFile:
            return
        self.ui.LE_load_config_ini.setText(inpFile)

    def read_config_ini(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        file_name = self.ui.LE_load_config_ini.text()
        self.conf = connect.Config(file_name)
        if len(self.conf.sections) == 1:
            self.ui.PB_connect_to_db.setVisible(True)
            # self.conf.credentials = 0
        elif len(self.conf.sections) > 1:
            self.ui.GB_dbs_available.setVisible(True)
            self.populate_available_dbs()

    def populate_available_dbs(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if self.GB_dbs_available.layout() is not None:
            self.deleteItemsOfLayout()
        vbox = QGridLayout()
        for i, variab in enumerate(self.conf.sections):
            radio = QRadioButton(variab)
            radio.index = i
            radio.toggled.connect(lambda: (self.ui.PB_connect_to_db.setVisible(True), self.set_section_ini()))
            vbox.addWidget(radio, i, 0)
        self.GB_dbs_available.setLayout(vbox)

    def set_section_ini(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.conf.credentials = self.sender().index

    def connect_to_db(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.window = guiSqlRM(self.conf)
        self.window.show()
        self.close()

    def deleteItemsOfLayout(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        layout_grupa = self.GB_dbs_available.layout()
        if layout_grupa is not None:
            while layout_grupa.count():
                item = layout_grupa.takeAt(0)
                widget = item.widget()
                if widget is not None:
                    widget.setParent(None)
                else:
                    self.deleteItemsOfLayout(item.layout())
            sip.delete(layout_grupa)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = window1()
    window.show()
    sys.exit(app.exec_())



import decimal
import os.path
import traceback
from PyQt5.QtWidgets import *
from PyQt5 import uic
from PyQt5 import QtCore, QtGui
from PyQt5.QtCore import *
import numpy as np
from datetime import datetime, timedelta
import datetime as dt
from dateutil.relativedelta import *
import sys
import os
from mysqlquerys import connect
from mysqlquerys import mysql_rm
np.set_printoptions(linewidth=250)
__version__ = 'V1'


class CheltPlanificate:
    def __init__(self, ini_file):
        # self.dataBase = connect.DataBase(db_type, data_base_name)
        self.ini_file = ini_file
        self.conf = connect.Config(self.ini_file)
        self.dataBase = self.sql_rm.DataBase(self.conf.credentials)
        # print(50*'Ö')
        # print(self.dataBase.allAvailableDatabases)
        # print(50*'Ö')

    @property
    def sql_rm(self):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if self.conf.db_type == 'mysql':
            sql_rm = mysql_rm
        return sql_rm

    def get_all_sql_vals(self, tableHead):
        # print(sys._getframe().f_code.co_name, tableHead)
        table_documente = mysql_rm.Table(self.conf.credentials, 'documente')
        all_docs_databases = table_documente.returnColumn('Name')
        all_chelt = []
        for db in all_docs_databases:
            print('ÄÄ', db, self.conf.credentials)
            new_conf = self.conf.credentials
            new_conf['database'] = db
            if db in self.dataBase.allAvailableDatabases:
                database = self.sql_rm.DataBase(new_conf)
                print('**', database.allAvailableTablesInDatabase)
        # sys.exit()



                # for table in self.dataBase.tables:
                # for table in self.dataBase.allAvailableTablesInDatabase:
                for table in database.allAvailableTablesInDatabase:
                    # self.dataBase.active_table = table
                    print('table', table)
                    # active_table = mysql_rm.Table(self.conf.credentials, table)
                    active_table = mysql_rm.Table(new_conf, table)
                    check = all(item in list(active_table.columnsProperties.keys()) for item in tableHead)
                    if check:
                        vals = active_table.returnColumns(tableHead)
                        for row in vals:
                            row = list(row)
                            row.insert(0, table)
                            all_chelt.append(row)

                newTableHead = ['table']
                for col in tableHead:
                    newTableHead.append(col)

        return newTableHead, all_chelt

    def excludeNone(self, tableHead, table):
        if None in table[:, tableHead.index('valid_to')]:
            newTable = table[table[:, tableHead.index('valid_to')] != np.array(None)]
        return tableHead, newTable

    def get_days_to_expire(self, tableHead, table):
        print(sys._getframe().f_code.co_name, tableHead)

        newTableHead = []
        for col in tableHead:
            newTableHead.append(col)
        newTableHead.append('days2Expire')

        newCol = np.empty((table.shape[0], 1), dtype=object)
        newCol.fill('')
        table = np.append(table, newCol, 1)
        for row in table:
            expDate = row[tableHead.index('valid_to')]
            today = dt.date.today()
            if expDate is not None:
                time2exp = (expDate - today).days
                row[-1] = time2exp
        return newTableHead, table

    def hideExpired(self, tableHead, table):
        print(sys._getframe().f_code.co_name, tableHead)
        newTable = table[table[:, tableHead.index('days2Expire')] > 0]
        return tableHead, newTable

    def relevantOnly(self, tableHead, table, days):
        print(sys._getframe().f_code.co_name, tableHead)
        colIndx = tableHead.index('path')
        indxPath = [x for x, item in enumerate(table[:, colIndx]) if (item == '' or item is None)]
        colIndx = tableHead.index('days2Expire')
        indxExp = [x for x, item in enumerate(table[:, colIndx]) if item != '' and 0<item<days]
        indx = [*indxPath, *indxExp]
        indx = list(set(indx))
        newTable = table[indx]
        return tableHead, newTable


class MyApp(QMainWindow):
    def __init__(self):
        super(MyApp, self).__init__()
        path2src, pyFileName = os.path.split(__file__)
        uiFileName = 'check_expiration.ui'
        path2GUI = os.path.join(path2src, 'GUI', uiFileName)
        Ui_MainWindow, QtBaseClass = uic.loadUiType(path2GUI)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        title = '{}_{}'.format(pyFileName, __version__)
        self.setWindowTitle(title)
        self.expDays = int(self.ui.SB_expDays.text())

        self.ini_file = r"D:\Python\MySQL\myfolderstructure.ini"
        self.data_base_name = 'myfolderstructure'

        self.cheltPlan = CheltPlanificate(self.ini_file)
        self.tableHead = ['id', 'name', 'valid_from', 'valid_to', 'auto_ext', 'path']
        self.newRows = []
        self.pathUpdates = []

        self.ui.planTable.horizontalHeader().sectionClicked.connect(self.sortPlan)
        self.ui.planTable.horizontalHeader().sectionDoubleClicked.connect(self.setFilter)
        self.ui.PB_update.clicked.connect(self.updateSQL)
        self.ui.PB_ResetFilter.clicked.connect(self.reset_Filter)
        self.prepareTablePlan()

        self.ui.CB_hideExpired.stateChanged.connect(self.prepareTablePlan)
        self.ui.CB_relOnly.stateChanged.connect(self.prepareTablePlan)
        self.ui.CB_excludeNone.stateChanged.connect(self.prepareTablePlan)
        self.ui.planTable.cellDoubleClicked.connect(self.updatePath)
        self.ui.planTable.cellChanged.connect(self.onCellChanged)

    def prepareTablePlan(self):
        print(sys._getframe().f_code.co_name)
        tableHead, table = self.cheltPlan.get_all_sql_vals(self.tableHead)
        table = np.atleast_2d(table)
        if self.ui.CB_excludeNone.isChecked():
            tableHead, table = self.cheltPlan.excludeNone(tableHead, table)
        tableHead, table = self.cheltPlan.get_days_to_expire(tableHead, table)
        if self.ui.CB_hideExpired.isChecked():
            tableHead, table = self.cheltPlan.excludeNone(tableHead, table)
            tableHead, table = self.cheltPlan.hideExpired(tableHead, table)
        elif self.ui.CB_relOnly.isChecked():
            tableHead, table = self.cheltPlan.relevantOnly(tableHead, table, self.expDays)

        self.populateExpensesPlan(tableHead, table)
        self.colorCells(tableHead)

    def populateExpensesPlan(self, tableHead, table):
        print(sys._getframe().f_code.co_name)
        self.ui.planTable.clear()
        self.ui.planTable.setColumnCount(len(tableHead))
        self.ui.planTable.setHorizontalHeaderLabels(tableHead)
        self.ui.planTable.setRowCount(table.shape[0])
        txt = 'No. of rows: {}'.format(table.shape[0])
        self.ui.label.setText(txt)
        for row in range(table.shape[0]):
            for col in range(table.shape[1]):
                if isinstance(table[row, col], int) or isinstance(table[row, col], float):
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, table[row, col])
                    self.ui.planTable.setItem(row, col, item)
                elif tableHead[col] == 'path' and str(table[row, col]) != '':
                    pth = table[row, col]
                    # print(20*'-', pth)
                    if pth:
                        pth = pth.strip('"')
                        if os.path.exists(pth):
                            url = bytearray(QUrl.fromLocalFile(pth).toEncoded()).decode()
                            print(20*'U', url)
                            print(20*'P', pth)
                            item = QLabel()
                            text = "<a href={}>{}</a>".format(url, pth)
                            item.setText(text)
                            item.setOpenExternalLinks(True)
                            self.ui.planTable.setCellWidget(row, col, item)
                        else:
                            item = QTableWidgetItem(str(table[row, col]))
                            self.ui.planTable.setItem(row, col, item)
                    else:
                        item = QTableWidgetItem(str(table[row, col]))
                        self.ui.planTable.setItem(row, col, item)
                elif tableHead[col] == 'path' and str(table[row, col]) == '':
                    item = QTableWidgetItem(str(table[row, col]))
                    self.ui.planTable.setItem(row, col, item)
                else:
                    item = QTableWidgetItem(str(table[row, col]))
                    self.ui.planTable.setItem(row, col, item)

    def updateSQL(self):
        # self.readTable()
        self.newRows = np.atleast_2d(self.newRows)
        self.newRows = np.unique(self.newRows, axis=0)
        print(self.newRows)
        print(self.newRows.shape)

        if self.newRows.shape[1] > 0:
            for row in self.newRows:
                tableName, id = row
                table = connect.Table(self.ini_file, self.data_base_name, tableName)
                matches = ('id', id)
                row = table.returnRowsWhere(matches)[0]

                cols = []
                vals = []
                for i, col in enumerate(table.columnsNames):
                    if col == 'id' or col == 'path':
                        continue
                    cols.append(col)
                    vals.append(row[i])

                newRowId = table.add_row(cols, vals)
                valid_to = row[table.columnsNames.index('valid_to')]
                freq = row[table.columnsNames.index('freq')]
                newValid_to = valid_to + relativedelta(months=freq)
                newValid_from = valid_to + relativedelta(days=1)
                table.changeCellContent('valid_to', newValid_to, 'id', newRowId)
                table.changeCellContent('valid_from', newValid_from, 'id', newRowId)
                table.changeCellContent('auto_ext', 1, 'id', newRowId)
                table.changeCellContent('auto_ext', 0, 'id', id)
        for row in self.pathUpdates:
            tableName, id, path = row
            table = connect.Table(self.ini_file, self.data_base_name, tableName)
            # matches = ('id', id)
            # row = table.returnRowsWhere(matches)[0]
            # newRowId = table.add_row(table.columnsNames[1:], row[1:])
            # valid_to = row[table.columnsNames.index('valid_to')]
            # freq = row[table.columnsNames.index('freq')]
            # newValid_to = valid_to + relativedelta(months=freq)
            # newValid_from = valid_to + relativedelta(days=1)
            # table.changeCellContent('valid_to', newValid_to, 'id', newRowId)
            # table.changeCellContent('valid_from', newValid_from, 'id', newRowId)
            table.changeCellContent('path', path, 'id', id)
        self.prepareTablePlan()

    def readTable(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        rows = self.ui.planTable.rowCount()
        cols = self.ui.planTable.columnCount()
        tableHead = []
        tableData = np.empty((rows, cols), dtype=object)
        for column in range(cols):
            colName = self.ui.planTable.horizontalHeaderItem(column).text()
            tableHead.append(colName)
            for row in range(rows):
                cell = self.ui.planTable.item(row, column)
                if colName == 'path':
                    print('+++++', colName, type(cell))
                    widget = self.ui.planTable.cellWidget(row, column)
                    print('----', widget)
                    if widget is not None:
                        pth = widget.text()
                        print('&&&&&', pth)
                        print('&&&&&', type(pth))
                        newPTH = os.path.abspath(os.path.expanduser(pth))
                        newPTH = newPTH.strip()
                        print('gggggg', newPTH)

                if cell is None:
                    tableData[row, column] = None
                else:
                    tableData[row, column] = cell.text()
                    # if colName == 'path':
                    #     print('+++++', colName, type(cell))
                    #     widget = self.ui.planTable.cellWidget(row, column)
                    #     if widget:
                    #         print('BINGO')
                    #         print(widget.text())

        return tableHead, tableData

    def colorCells(self, tableHead):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        rows = self.ui.planTable.rowCount()
        cols = self.ui.planTable.columnCount()
        if 'newRow' not in tableHead:
            tableHead.append('newRow')
            self.ui.planTable.insertColumn(cols)
        cols = self.ui.planTable.columnCount()
        self.ui.planTable.setHorizontalHeaderLabels(tableHead)
        for row in range(rows):
            for column in range(cols):
                cell = self.ui.planTable.item(row, column)
                headerName = self.ui.planTable.horizontalHeaderItem(column).text()
                if headerName == 'days2Expire':
                    if cell.text():
                        days2Expire = int(cell.text())
                        if self.expDays > days2Expire > 0:
                            self.ui.planTable.item(row, column).setBackground(QtGui.QColor('red'))
                elif headerName == 'path':
                    widget = self.ui.planTable.cellWidget(row, column)
                    if widget is None:
                        self.ui.planTable.item(row, column).setBackground(QtGui.QColor('red'))
                elif headerName == 'newRow':
                    cell = QTableWidgetItem()
                    cell.setFlags(QtCore.Qt.ItemIsUserCheckable |QtCore.Qt.ItemIsEnabled)
                    cell.setCheckState(QtCore.Qt.Unchecked)
                    # cell.stateChanged.connect(self.clickBoxStateChanged(col))
                    self.ui.planTable.setItem(row, column, cell)

    def onCellChanged(self, row, column):
        item = self.ui.planTable.item(row, column)
        if item.checkState() == 2:
        # if self.ui.planTable.cellWidget(row, column).isChecked():
            cellTable = self.ui.planTable.item(row, 0)
            cellID = self.ui.planTable.item(row, 1)
            tup = (cellTable.text(), cellID.text())
            self.newRows.append(tup)

    def updatePath(self, row, col):
        print(sys._getframe().f_code.co_name)
        headerName = self.ui.planTable.horizontalHeaderItem(col).text()
        if headerName == 'path':
            inpFile, _ = QFileDialog.getOpenFileName(None, 'Select file', '', '')
            if inpFile:
                if os.path.exists(inpFile):
                    item = QTableWidgetItem(str(inpFile))
                    self.ui.planTable.setItem(row, col, item)
                    self.ui.planTable.item(row, col).setBackground(QtGui.QColor('green'))
                    cols = self.ui.planTable.columnCount()
                    for col in range(cols):
                        headerName = self.ui.planTable.horizontalHeaderItem(col).text()
                        if headerName == 'table':
                            table = self.ui.planTable.item(row, col).text()
                        elif headerName == 'id':
                            id = self.ui.planTable.item(row, col).text()
                    tup = (table, id, inpFile)
                    self.pathUpdates.append(tup)

    def sortPlan(self, logical_index):
        print(sys._getframe().f_code.co_name)
        header = self.ui.planTable.horizontalHeader()
        order = Qt.DescendingOrder
        if not header.isSortIndicatorShown():
            header.setSortIndicatorShown(True)
        elif header.sortIndicatorSection() == logical_index:
            order = header.sortIndicatorOrder()
        header.setSortIndicator(logical_index, order)
        self.ui.planTable.sortItems(logical_index, order)

    def setFilter(self, logical_index):
        print(sys._getframe().f_code.co_name)
        # colName = self.ui.planTable.columnsNames[logical_index]
        colName = self.ui.planTable.horizontalHeaderItem(logical_index).text()
        header = self.ui.planTable.horizontalHeader()

        geom = QtCore.QRect(header.sectionViewportPosition(logical_index), 0, header.sectionSize(logical_index),
                            header.height())
        item = QLineEdit(header)
        item.setGeometry(geom)
        item.show()
        item.setFocus()
        item.editingFinished.connect(lambda: (self.applyFilter(colName, item.text()),
                                              item.clear(),
                                              item.hide(),
                                              item.deleteLater()))

    def applyFilter(self, colName, filter):
        print(sys._getframe().f_code.co_name)
        if filter == '':
            return

        tableHead, tableData = self.readTable()

        filterText = self.ui.lineEditFilterList.text()
        if not filterText:
            if isinstance(filter, str):
                filterText += '{}="{}"'.format(colName, filter)
            elif isinstance(filter, tuple):
                filterText += '{} < {} < {}'.format(filter[0], colName, filter[1])
            elif isinstance(filter, list):
                filterText += '{} in {}"'.format(str(filter), colName)
        else:
            if isinstance(filter, str):
                filterText += '; {}="{}"'.format(colName, filter)
            elif isinstance(filter, tuple):
                filterText += '; {} < {} < {}'.format(filter[0], colName, filter[1])
            elif isinstance(filter, list):
                filterText += '; {} in {}"'.format(str(filter), colName)

        self.ui.lineEditFilterList.setText(filterText)

        newTableData = tableData[np.where(tableData[:, tableHead.index(colName)] == filter)]
        self.populateExpensesPlan(tableHead, newTableData)
        self.colorCells(tableHead)

    def reset_Filter(self):
        self.ui.lineEditFilterList.setText('')
        self.prepareTablePlan()


class FilterWindow(QDialog):
    def __init__(self, colType, colName):
        super(FilterWindow, self).__init__()
        path2src, pyFileName = os.path.split(__file__)
        uiFileName = 'filterWindow.ui'
        path2GUI = os.path.join(path2src, 'GUI', uiFileName)
        Ui_MainWindow, QtBaseClass = uic.loadUiType(path2GUI)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.colName = colName
        self.ui.GB_DateInterval.setVisible(False)
        self.ui.GB_IntInterval.setVisible(False)
        self.ui.checkBoxDateInterval.stateChanged.connect(self.openDateInterval)
        self.ui.checkBoxIntInterval.stateChanged.connect(self.openIntInterval)

        self.rejected.connect(self.byebye)

        if colType == 'int':
            self.ui.GB_IntInterval.setVisible(True)
            self.ui.lineEdit_max.setEnabled(False)
        if colType == 'date':
            self.ui.GB_DateInterval.setVisible(True)
            self.ui.dateEditTo.setEnabled(False)

    def openIntInterval(self):
        if self.ui.checkBoxIntInterval.isChecked():
            self.ui.lineEdit_max.setEnabled(True)
            self.ui.label_int.setText('< {} <'.format(self.colName))
        else:
            self.ui.lineEdit_max.setEnabled(False)
            self.ui.label_int.setText('= {}'.format(self.colName))

    def openDateInterval(self):
        if self.ui.checkBoxDateInterval.isChecked():
            self.ui.dateEditTo.setEnabled(True)
            self.ui.label_date.setText('< {} <'.format(self.colName))
        else:
            self.ui.dateEditTo.setEnabled(False)
            self.ui.label_date.setText('= {}'.format(self.colName))

    def byebye(self):
        self.close()

    def intInterval(self):
        if self.ui.checkBoxIntInterval.isChecked():
            tup = (self.ui.lineEdit_min.text(), self.ui.lineEdit_max.text())
            return tup
        else:
            return self.ui.lineEdit_min.text()

    def dateInterval(self):
        if self.ui.checkBoxDateInterval.isChecked():
            tup = (self.ui.dateEditFrom.date(), self.ui.dateEditTo.date())
            return tup
        else:
            return self.ui.dateEditFrom.date()

    @staticmethod
    def getIntInterval(colName):
        dialog = FilterWindow('int', colName)
        result = dialog.exec_()
        filt = dialog.intInterval()
        if result == QDialog.Accepted:
            return filt
        else:
            return None

    @staticmethod
    def getDateInterval(colName):
        dialog = FilterWindow('date', colName)
        result = dialog.exec_()
        filt = dialog.dateInterval()
        if result == QDialog.Accepted:
            return filt
        else:
            return None


class InsertNewRow(QDialog):
    def __init__(self, cols):
        super(InsertNewRow, self).__init__()
        self.cols = cols
        path2src, pyFileName = os.path.split(__file__)
        uiFileName = 'addNewRow.ui'
        path2GUI = os.path.join(path2src, 'GUI', uiFileName)
        Ui_MainWindow, QtBaseClass = uic.loadUiType(path2GUI)
        self.gui = Ui_MainWindow()
        self.gui.setupUi(self)
        self.fillInTable()

    def fillInTable(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.gui.tableWidget.setRowCount(len(self.cols))
        for row, colName in enumerate(self.cols):
            cell = QLineEdit()
            cell.setText(str(colName))
            cell.setEnabled(False)
            self.gui.tableWidget.setCellWidget(row, 0, cell)

            colType, null, key, default, extra = self.cols[colName][:5]
            cell = QLineEdit()
            cell.setText(str(colType))
            cell.setEnabled(False)
            self.gui.tableWidget.setCellWidget(row, 1, cell)
            # self.gui.tableWidget.setItem(row, 1, QTableWidgetItem(colType))
            cell = QLineEdit()
            cell.setText(str(null))
            cell.setEnabled(False)
            self.gui.tableWidget.setCellWidget(row, 2, cell)
            # self.gui.tableWidget.setItem(row, 2, QTableWidgetItem(null))
            cell = QLineEdit()
            cell.setText(str(key))
            cell.setEnabled(False)
            self.gui.tableWidget.setCellWidget(row, 3, cell)
            # self.gui.tableWidget.setItem(row, 3, QTableWidgetItem(key))
            cell = QLineEdit()
            cell.setText(str(default))
            cell.setEnabled(False)
            self.gui.tableWidget.setCellWidget(row, 4, cell)
            # self.gui.tableWidget.setItem(row, 4, QTableWidgetItem(default))

            cellAutoIncr = QTableWidgetItem()
            cellAutoIncr.setFlags(QtCore.Qt.ItemIsEnabled)#QtCore.Qt.ItemIsUserCheckable |

            if 'auto_increment' in extra:
                autoIncremVal = self.cols[colName][-1]
                cell = QLineEdit()
                cell.setText(str(autoIncremVal))
                cell.setEnabled(False)
                cellAutoIncr.setCheckState(QtCore.Qt.Checked)
                self.gui.tableWidget.setCellWidget(row, 6, cell)
            else:
                cellAutoIncr.setCheckState(QtCore.Qt.Unchecked)
                self.gui.tableWidget.setItem(row, 6, QTableWidgetItem(extra))
            self.gui.tableWidget.setItem(row, 5, cellAutoIncr)

            if 'timestamp' in colType:
                cell = QDateEdit()
                cell.setDisplayFormat('yyyy-MM-dd hh:mm:ss')
                cell.setDateTime(QDateTime(datetime.now().year,
                                           datetime.now().month,
                                           datetime.now().day,
                                           datetime.now().hour,
                                           datetime.now().minute,
                                           datetime.now().second))
                cell.setEnabled(False)
                self.gui.tableWidget.setCellWidget(row, 6, cell)

            if 'date' in colType:
                cell = QDateEdit()
                cell.setDisplayFormat('yyyy-MM-dd')
                cell.setDate(QDate(datetime.now().year,
                                           datetime.now().month,
                                           datetime.now().day))
                # cell.setEnabled(False)
                self.gui.tableWidget.setCellWidget(row, 6, cell)
            elif ('smallint' in colType or 'tinyint' in colType) and 'active' in colName:
                cell = QTableWidgetItem()
                cell.setFlags(QtCore.Qt.ItemIsUserCheckable | QtCore.Qt.ItemIsEnabled)
                cell.setCheckState(QtCore.Qt.Checked)
                self.gui.tableWidget.setCellWidget(row, 6, cell)

    def readTable(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        rows = self.gui.tableWidget.rowCount()
        cols = self.gui.tableWidget.columnCount()

        columns = []
        values = []

        for row in range(rows):
            for column in range(cols):
                cell = self.gui.tableWidget.item(row, column)
                headerName = self.gui.tableWidget.horizontalHeaderItem(column).text()
                if headerName == 'ColName':
                    # colName = cell.text()
                    widget = self.gui.tableWidget.cellWidget(row, column)
                    colName = widget.text()
                if headerName == 'Value':
                    widget = self.gui.tableWidget.cellWidget(row, column)
                    if cell is not None:
                        if cell.text():
                            colValue = cell.text()
                            print(colValue)
                        elif cell.checkState() == QtCore.Qt.Checked:
                            colValue = 1
                        elif cell.checkState() == QtCore.Qt.Unchecked:
                            colValue = 0
                        columns.append(colName)
                        values.append(colValue)
                    elif widget:
                        if isinstance(widget, QDateTimeEdit):
                            colValue = widget.dateTime()
                            colValue = colValue.toPyDateTime()
                            columns.append(colName)
                            values.append(str(colValue))
        return tuple(columns), tuple(values)

    @staticmethod
    def getNewRowValues(cols):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        dialog = InsertNewRow(cols)
        result = dialog.exec_()
        cols, values = dialog.readTable()
        if result == QDialog.Accepted:
            return cols, values
        else:
            return None


def main():
    app = QApplication(sys.argv)
    window = MyApp()
    window.show()
    app.exec_()


if __name__ == '__main__':
    main()



import numpy as np
import shutil
import rappmysql
from rappmysql.mysqlquerys import connect
from rappmysql.mysqlquerys import mysql_rm
from datetime import datetime, timedelta, date
from dateutil.relativedelta import relativedelta
import traceback
import sys, os
import time
import pathlib
import zipfile

np.set_printoptions(linewidth=250)
compName = os.getenv('COMPUTERNAME')
# try:
#     compName = os.getenv('COMPUTERNAME')
#     if compName == 'DESKTOP-5HHINGF':
#         ini_users = r"D:\Python\MySQL\users.ini"
#         ini_chelt = r"D:\Python\MySQL\cheltuieli.ini"
#         ini_masina = r"D:\Python\MySQL\masina.ini"
#         report_dir = r"D:\Python\MySQL\onlineanywhere\static"
#     else:
#         ini_users = r"C:\_Development\Diverse\pypi\cfgm.ini"
#         ini_chelt = r"C:\_Development\Diverse\pypi\cfgm.ini"
#         ini_masina = r"C:\_Development\Diverse\pypi\cfgm.ini"
#         # ini_file = r"C:\_Development\Diverse\pypi\cfgmRN.ini"
#         report_dir = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\static"
# except:
#     ini_users = '/home/radum/mysite/static/wdb.ini'

app_masina_tables = {'all_cars': 'id_users', 'masina': 'id_users'}
app_masina_gui = ['masina.ui', 'add_auto.ui']


class CheckAutoRequiredFiles:
    def __init__(self, ini_file):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        if isinstance(ini_file, dict):
            self.credentials = ini_file
        else:
            self.conf = connect.Config(ini_file)
            self.credentials = self.conf.credentials
        self.auto_db = None
        path2GUI = pathlib.Path(__file__)
        self.path2GUI = path2GUI.resolve(path2GUI).parent / 'GUI'
        self.pth2SQLtables = os.path.join(os.path.dirname(__file__), 'static', 'sql')
        # self.all_cars_table = mysql_rm.Table(self.credentials, 'all_cars')
        # self.alimentari = mysql_rm.Table(self.credentials, 'masina')
        self.check_requirements()

    def check_requirements(self):
        self.check_connection_to_database()
        self.check_sql_gui_files_existance()
        self.check_tabels_in_database()

    def check_connection_to_database(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(
            __name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        if not isinstance(self.credentials, dict):
            raise RuntimeError('Credentials not dict')
        self.auto_db = mysql_rm.DataBase(self.credentials)
        if not self.auto_db.is_connected:
            raise RuntimeError('Could not connect to database')
        print('Connected to auto_db:', self.auto_db.is_connected)

    def check_sql_gui_files_existance(self, ):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        for table in app_masina_tables.keys():
            sql_file = os.path.join(self.pth2SQLtables, '{}_template.sql'.format(table))
            print('##sql_table_template##', sql_file)
            if not os.path.exists(sql_file):
                raise FileNotFoundError('{}'.format(sql_file))

        for gui in app_masina_gui:
            sql_file = os.path.join(self.path2GUI, gui)
            print('##gui_file##', sql_file)
            if not os.path.exists(sql_file):
                raise FileNotFoundError('{}'.format(sql_file))

    def check_tabels_in_database(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        for table in app_masina_tables.keys():
            # print('##table##', table)
            sql_file = os.path.join(self.pth2SQLtables, '{}_template.sql'.format(table))
            if table not in self.auto_db.allAvailableTablesInDatabase:
                print('Table {} not in database...creating it'.format(table))
                self.auto_db.createTableFromFile(sql_file, table)
            else:
                # exec("sqltable = mysql_rm.Table(self.credentials, '{}')".format(table))
                varName = 'table_{}'.format(table)
                # print("{} = mysql_rm.Table(self.credentials, '{}')".format(varName, table))
                # print('ÄÄ', varName)
                loc = locals()
                # print(loc)
                exec("{} = mysql_rm.Table(self.credentials, '{}')".format(varName, table), globals(), loc)
                varName = loc[varName]
                same = varName.compare_sql_file_to_sql_table(sql_file)
                if same is not True:
                    print(same)


class AutoApp:
    def __init__(self, ini_masina, user_id):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # super().__init__(ini_masina)
        self.id = user_id
        if isinstance(ini_masina, dict):
            self.credentials = ini_masina
        else:
            self.conf = connect.Config(ini_masina)
            self.credentials = self.conf.credentials
        self.auto_db = mysql_rm.DataBase(self.credentials)
        self.all_cars_table = mysql_rm.Table(self.credentials, 'all_cars')
        self.alimentari = mysql_rm.Table(self.credentials, 'masina')

    @property
    def masini(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        masini = {}
        matches = ('id_users', self.id)
        cars_rows = self.all_cars_table.returnRowsWhere(matches)
        if cars_rows:
            for row in cars_rows:
                print('***row', row)
                indx_id = self.all_cars_table.columnsNames.index('id')
                indx_brand = self.all_cars_table.columnsNames.index('brand')
                indx_model = self.all_cars_table.columnsNames.index('model')
                table_name = '{}_{}'.format(row[indx_brand], row[indx_model])
                masini[row[indx_id]] = table_name
        return masini

    @property
    def electric_providers(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        matches = [('type', 'electric')]
        col = self.alimentari.returnCellsWhere('eProvider', matches)
        electric_providers = list(set(col))
        # electric_providers = ['eCharge', 'MyHyundai', 'EnBW', 'SWM_Plus', 'SWM']
        return electric_providers

    def add_car(self, brand, model, car_type):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        brand = brand.lower()
        model = model.lower()
        car_type = car_type.lower()
        cols = ('id_users', 'brand', 'model', 'cartype')
        vals = (self.id, brand, model, car_type)
        matches = [('id_users', self.id), ('brand', brand), ('model', model), ('cartype', car_type)]
        existing_row = self.all_cars_table.returnRowsWhere(matches)
        # print('existing_row', existing_row)
        if existing_row:
            print('car already existing at id {}'.format(existing_row[0][0]))
            return
        else:
            self.all_cars_table.addNewRow(cols, vals)
            # new_auto_table = '{}_{}'.format(brand, model)
            # new_auto_table = new_auto_table.lower()
            # if new_auto_table in self.auto_db.allAvailableTablesInDatabase:
            #     print('table {} existing in database'.format(new_auto_table))
            # else:
            #     pth_auto_template = os.path.join(os.path.dirname(__file__), 'static', 'sql', 'auto_template.sql')
            #     self.auto_db.createTableFromFile(pth_auto_template, new_auto_table)

    def export_car_sql(self, car_id, export_files=False):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        all_cars_ident = {'id_users': self.id, 'id': car_id}
        masina_ident = {'id_users': self.id, 'id_all_cars': car_id}
        tables = {'all_cars': all_cars_ident,
                  'masina': masina_ident}

        tim = datetime.strftime(datetime.now(), '%Y_%m_%d__%H_%M_%S')
        output_dir = os.path.join(os.path.dirname(__file__), 'static', 'backup_profile',
                                  '{:09d}'.format(self.id),
                                  '{:09d}'.format(car_id),
                                  '{}_{:09d}'.format(tim, car_id))
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        if export_files:
            sql_query = self.auto_db.return_sql_text(tables, export_files=output_dir)
        else:
            sql_query = self.auto_db.return_sql_text(tables)

        output_sql_file = os.path.join(output_dir, '{}_{:09d}.sql'.format(tim, self.id))
        FILE = open(output_sql_file, "w", encoding="utf-8")
        FILE.writelines(sql_query)
        FILE.close()
        #####
        output_zip = os.path.join(os.path.dirname(output_dir), '{}.zip'.format(output_dir))
        zip_file = self.zip_profile_files(output_dir, output_zip)
        if os.path.exists(zip_file):
            shutil.rmtree(output_dir)
        print('finished backup')

        return output_sql_file

    def import_car_with_files(self, zip_file, import_files=False):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        output_dir, file = os.path.split(zip_file)
        src_dir = self.unzip_profile_files(zip_file, output_dir)
        src_dir = os.path.join(src_dir, file[:-4])
        if not os.path.exists(src_dir):
            raise RuntimeError('Missing Folder {}'.format(src_dir))

        sql_files = [x for x in os.listdir(src_dir) if x.endswith('.sql')]
        sql_file = os.path.join(src_dir, sql_files[0])
        # print(sql_file)
        # return
        self.auto_db.run_sql_file(sql_file)
        if import_files:
            attachments = [x for x in os.listdir(src_dir) if
                           (x.endswith('.jpg') or
                            x.endswith('.pdf') or
                            x.endswith('.csv') or
                            x.endswith('.CSV')
                            )]
            tab = []
            for file_name in attachments:
                try:
                    # print(file_name)
                    table_id, orig_name = file_name.split('+')
                    fil = os.path.join(src_dir, file_name)
                    self.alimentari.changeCellContent('file', fil, 'id', table_id)
                    self.alimentari.changeCellContent('file_name', str(orig_name), 'id', table_id)
                    # print(id_users, table_name, table_id, orig_name)
                except:
                    print('could not import {}, name not ok'.format(file_name))
                tup = (table_id, orig_name, file_name)
                tab.append(tup)
            # tab = np.atleast_2d(tab)
            # all_sql_tables = list(np.unique(tab[:, 1]))
            # for table_name in all_sql_tables:
            #     # print('table_name', table_name)
            #     sql_table = mysql_rm.Table(self.credentials, table_name)
            #     table = tab[tab[:, 1] == table_name]
            #     for row in table:
            #         id_users, table_name, table_id, orig_name, fl_name = row
            #         # print('&', id_users, table_name, table_id, orig_name, fl_name)
            #         fil = os.path.join(src_dir, fl_name)
            #         sql_table.changeCellContent('file', fil, 'id', table_id)
            #         sql_table.changeCellContent('file_name', str(orig_name), 'id', table_id)
        if os.path.exists(src_dir):
            shutil.rmtree(src_dir)

    def get_id_all_cars(self, table_name):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        brand, model = table_name.split('_')
        matches = [('id_users', self.id),
                   ('brand', brand),
                   ('model', model),
                   ]
        # print(matches)
        id_all_cars = self.all_cars_table.returnCellsWhere('id', matches)[0]
        return id_all_cars

    def delete_auto(self, table_name):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        brand, model = table_name.split('_')
        matches = [('id_users', self.id), ('brand', brand), ('model', model)]
        id_car = self.all_cars_table.returnCellsWhere('id', matches)
        # print('id_car', id_car)
        condition = ['id', id_car[0]]
        self.all_cars_table.delete_multiple_rows(condition)

    def erase_autoapp_traces(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # for i in self.chelt_db.checkProcess():
        #     print(i)
        self.auto_db.killAllProcess()
        self.auto_db.drop_table_list(list(app_masina_tables.keys()))

    def unzip_profile_files(self, src_file, output_dir):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        with zipfile.ZipFile(src_file, 'r') as zip_ref:
            zip_ref.extractall(output_dir)
        return output_dir

    def zip_profile_files(self, src_dir, output_file):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        relroot = os.path.abspath(os.path.join(src_dir, os.pardir))
        with zipfile.ZipFile(output_file, "w", zipfile.ZIP_DEFLATED) as zip:
            for root, dirs, files in os.walk(src_dir):
                # add directory (needed for empty dirs)
                zip.write(root, os.path.relpath(root, relroot))
                for file in files:
                    filename = os.path.join(root, file)
                    if os.path.isfile(filename):  # regular files only
                        arcname = os.path.join(os.path.relpath(root, relroot), file)
                        zip.write(filename, arcname)
        return output_file


class Masina:
    def __init__(self, ini_file, id_users, id_car):
        '''
        :param ini_file:type=QFileDialog.getOpenFileName name=filename file_type=(*.ini;*.txt)
        '''
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # super().__init__(ini_file)
        self.id_users = id_users
        self.id_car = id_car
        if isinstance(ini_file, dict):
            self.credentials = ini_file
        else:
            self.conf = connect.Config(ini_file)
            self.credentials = self.conf.credentials
        self.auto_db = mysql_rm.DataBase(self.credentials)
        self.all_cars_table = mysql_rm.Table(self.credentials, 'all_cars')
        self.alimentari = mysql_rm.Table(self.credentials, 'masina')

        # if isinstance(ini_file, dict):
        #     credentials = ini_file
        # else:
        #     self.conf = connect.Config(ini_file)
        #     credentials = self.conf.credentials
        # self.checkup_list(credentials)
        # self.alimentari = mysql_rm.Table(self.credentials, 'masina')
        self.types_of_costs = ["electric", "benzina", "intretinere", "asigurare", 'impozit', 'TüV', 'carwash', 'other']
        self.no_of_records = self.get_no_of_records()
        self.total_money = self.get_total_money()
        self.tot_benzina = self.get_tot_benzina()
        self.tot_electric = self.get_tot_electric()
        self.monthly_benzina = self.get_monthly_benzina()
        self.monthly_electric = self.get_monthly_electric()
        self.monthly = self.get_monthly()
        self.db_start_date = self.get_db_start_date()
        self.db_last_record_date = self.get_db_last_record_date()
        self.table_alimentari = self.get_table_alimentari()
        self.dict_totals = self.get_dict_totals()
        self.table_totals = self.get_table_totals()
        self.last_records = self.get_last_records()

    # @property
    def get_no_of_records(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        return self.alimentari.noOfRows

    @property
    def default_interval(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        startDate = datetime(datetime.now().year - 1, datetime.now().month, datetime.now().day)
        endDate = datetime(datetime.now().year, datetime.now().month, datetime.now().day)
        return startDate, endDate

    # @property
    def get_total_money(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        col = self.alimentari.returnColumn('brutto')
        return round(sum(col), 2)

    # @property
    def get_tot_benzina(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        matches = [('type', 'benzina'),
                   ('id_users', self.id_users),
                   ('id_all_cars', self.id_car)]
        col = self.alimentari.returnCellsWhere('brutto', matches)
        return round(sum(col), 2)

    # @property
    def get_tot_electric(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        matches = [('type', 'electric'),
                   ('id_users', self.id_users),
                   ('id_all_cars', self.id_car)]
        col = self.alimentari.returnCellsWhere('brutto', matches)
        return round(sum(col), 2)

    # @property
    def get_monthly(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        try:
            return round((self.monthly_benzina + self.monthly_electric), 2)
        except:
            return None

    # @property
    def get_monthly_benzina(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        try:
            matches = [('type', 'benzina'),
                       ('id_users', self.id_users),
                       ('id_all_cars', self.id_car)]
            rows = self.alimentari.returnRowsWhere(matches)
            rows = np.atleast_2d(rows)
            money = rows[:, self.alimentari.columnsNames.index('brutto')]
            start_date = min(rows[:, self.alimentari.columnsNames.index('data')])
            finish_date = max(rows[:, self.alimentari.columnsNames.index('data')])

            total_money = round(sum(money), 2)

            # print(start_date)
            # print(finish_date)
            # print(total_money)
            days = (finish_date - start_date).days
            # print(days)
            average_day_per_month = 365 / 12
            monthly = (average_day_per_month * total_money) / days
            return round(monthly, 2)
            # return round(5.5555, 2)
        except:
            return None

    # @property
    def get_monthly_electric(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        try:
            matches = [('type', 'electric'),
                       ('id_users', self.id_users),
                       ('id_all_cars', self.id_car)]
            rows = self.alimentari.returnRowsWhere(matches)
            # print(rows)
            # print(type(rows))
            if rows:
                rows = np.atleast_2d(rows)
                money = rows[:, self.alimentari.columnsNames.index('brutto')]
                start_date = min(rows[:, self.alimentari.columnsNames.index('data')])
                finish_date = max(rows[:, self.alimentari.columnsNames.index('data')])
                # print('*****', money, start_date, finish_date)
                total_money = round(sum(money), 2)
                days = (finish_date - start_date).days
                average_day_per_month = 365 / 12
                monthly = (average_day_per_month * total_money) / days
                return round(monthly, 2)
        except:
            return None

    # @property
    def get_db_start_date(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        matches = [('id_users', self.id_users),
                   ('id_all_cars', self.id_car)]
        all_dates = self.alimentari.returnCellsWhere('data', matches)
        # print('**all_dates', all_dates, type(all_dates))
        if all_dates:
            start_date = min(all_dates)
        else:
            start_date = None
        return start_date

    # @property
    def get_db_last_record_date(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        try:
            # all_dates = self.alimentari.returnColumn('data')
            matches = [('id_users', self.id_users),
                       ('id_all_cars', self.id_car)]
            all_dates = self.alimentari.returnCellsWhere('data', matches)
            finish_date = max(all_dates)
            return finish_date
        except:
            return None

    # @property
    def get_table_alimentari(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        arr = [('', 'Alimentari[€]', 'Benzina[€]', 'Electric[€]')]
        if self.no_of_records > 0:
            total_alim = round(self.tot_benzina + self.tot_electric, 2)
            arr.append(('Monthly', self.monthly, self.monthly_benzina, self.monthly_electric))
            arr.append(('Total', total_alim, self.tot_benzina, self.tot_electric))
        else:
            arr.append(('Monthly', None, None, None))
            arr.append(('Total', None, None, None))

        arr = np.atleast_2d(arr)
        return arr

    def dict_last_months(self, delta_mths=12):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        if not self.db_start_date:
            return None
        table_dict = {}
        try:
            for mnth in reversed(range(delta_mths+1)):
                dd = {}
                start_date = date.today() - relativedelta(months=+mnth)
                mnth_as_str = start_date.strftime("%B")
                startTime, endTime = self.get_monthly_interval(mnth_as_str, start_date.year)
                interval = '{}.{}'.format(mnth_as_str[:3], start_date.year)
                # print(startTime, endTime)
                tot = 0
                for t in self.types_of_costs:
                    matches = [('id_users', '=', self.id_users),
                               ('id_all_cars', '=', self.id_car),
                               ('type', '=', t),
                               ('data', '>=', startTime),
                               ('data', '<=', endTime)
                               ]
                    payments4Interval = self.alimentari.returnRowsQuery(matches)
                    if payments4Interval:
                        payments4Interval = np.atleast_2d(payments4Interval)
                        col = payments4Interval[:, self.alimentari.columnsNames.index('brutto')]
                        value = sum(col)
                        value = round(value, 2)
                        dd[t] = value
                        tot += value
                dd['total/row'] = round(tot, 2)
                table_dict[interval] = dd
            return table_dict
        except:
            return str(traceback.format_exc())

    # @property
    def get_dict_totals(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        if not self.db_start_date:
            return None
        table_dict = {}
        try:
            for year in reversed(range(self.db_start_date.year, self.db_last_record_date.year + 1)):
                dd = {}
                startTime = datetime(year, 1, 1)
                endTime = datetime(year, 12, 31)
                tot = 0
                for t in self.types_of_costs:
                    # print('t', t)
                    matches = [('id_users', '=', self.id_users),
                               ('id_all_cars', '=', self.id_car),
                               ('type', '=', t),
                               ('data', '>=', startTime),
                               ('data', '<=', endTime)
                               ]
                    payments4Interval = self.alimentari.returnRowsQuery(matches)
                    if payments4Interval:
                        payments4Interval = np.atleast_2d(payments4Interval)
                        col = payments4Interval[:, self.alimentari.columnsNames.index('brutto')]
                        value = sum(col)
                        value = round(value, 2)
                        dd[t] = value
                        tot += value
                dd['total/row'] = round(tot, 2)
                table_dict[year] = dd
            return table_dict
        except:
            return str(traceback.format_exc())

    # @property
    def get_table_totals(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        try:
            table_head = self.types_of_costs.copy()
            table_totals = []
            for year, expenses in self.dict_totals.items():
                row = [year]
                total_per_year = 0
                for col in table_head:
                    if col in expenses.keys():
                        row.append(expenses[col])
                        total_per_year += expenses[col]
                    else:
                        row.append(0)
                row.append(round(total_per_year, 2))
                table_totals.append(tuple(row))
            table_head.insert(0, 'year')
            table_head.append('tot/year')
            table_totals.insert(0, tuple(table_head))
            table_totals = np.atleast_2d(table_totals)
            return table_totals
        except:
            return None

    # @property
    def get_last_records(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # print(len(tuple(self.alimentari.columnsNames)))
        table_head = self.alimentari.columnsNames.copy()
        table_head.remove('file')
        last_records = [tuple(table_head)]
        for typ in self.types_of_costs:
            matches = [('id_users', self.id_users),
                       ('id_all_cars', self.id_car),
                       ('type', typ)]
            table = self.alimentari.filterRows(matches, order_by=('data', 'DESC'))
            if table:
                last_records.append(tuple(table[0]))
        last_records = np.atleast_2d(last_records)
        return last_records

    def delete_row(self, row_id):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        condition = ('id', row_id)
        self.alimentari.delete_multiple_rows(condition)

    def get_monthly_interval(self, month: str, year):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        mnth = datetime.strptime(month, "%B").month
        startDate = datetime(year, mnth, 1)

        if mnth != 12:
            lastDayOfMonth = datetime(year, mnth + 1, 1) - timedelta(days=1)
        else:
            lastDayOfMonth = datetime(year + 1, 1, 1) - timedelta(days=1)

        return startDate, lastDayOfMonth

    def get_all_alimentari(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        cols = []
        for k, v in self.alimentari.columnsDetProperties.items():
            if v[0] == 'longblob':
                continue
            cols.append(k)
        alimentari = self.alimentari.returnColumns(cols)
        # alimentari = self.alimentari.returnAllRecordsFromTable()
        alimentari = np.atleast_2d(alimentari)
        alimentari = np.insert(alimentari, 0, cols, axis=0)
        return alimentari

    def get_alimentari_for_interval_type(self, selectedStartDate, selectedEndDate, alim_type):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        matches = [('data', (selectedStartDate, selectedEndDate)),
                   ('id_users', self.id_users),
                   ('id_all_cars', self.id_car)]
        if alim_type:
            matches.append(('type', alim_type))
        print(matches)
        table = self.alimentari.filterRows(matches, order_by=('data', 'DESC'))

        if table:
            table_head = []
            for col_name, prop in self.alimentari.columnsDetProperties.items():
                # print(col_name, prop)
                if prop[0] == 'longblob':
                    continue
                table_head.append(col_name)
            arr = np.atleast_2d(table)
            arr = np.insert(arr, 0, np.array(table_head), axis=0)
        else:
            arr = np.atleast_2d(np.array(self.alimentari.columnsNames))
        return arr

    def upload_file(self, file_name, id):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.alimentari.changeCellContent('file', file_name, 'id', id)
        pth, file_name = os.path.split(file_name)
        self.alimentari.changeCellContent('file_name', file_name, 'id', id)

    def insert_new_alim(self, current_id_users, id_all_cars, data, alim_type, brutto, amount, refuel, other, recharges,
                        km, comment, file, provider):
        '''
        :param data:type=dateTime name=date
        :param alim_type:type=comboBox name=alim_type items=[electric,benzina,TüV,intretinere]
        :param brutto:type=text name=brutto
        :param amount:type=text name=amount
        :param refuel:type=text name=refuel
        :param other:type=text name=other
        :param recharges:type=text name=recharges
        :param km:type=text name=km
        :param comment:type=text name=comment
        :param file:type=QFileDialog.getOpenFileName name=file
        '''
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        if file:
            _, file_name = os.path.split(file)
            cols = ['id_users', 'id_all_cars', 'data', 'type', 'brutto', 'amount', 'refuel', 'other', 'recharges',
                    'ppu', 'km', 'comment', 'file', 'file_name', 'eProvider']
        else:
            cols = ['id_users', 'id_all_cars', 'data', 'type', 'brutto', 'amount', 'refuel', 'other', 'recharges',
                    'ppu', 'km', 'comment', 'eProvider']
        try:
            if isinstance(brutto, str) and ',' in brutto:
                brutto = brutto.replace(',', '.')
            brutto = float(brutto)
        except:
            brutto = None
        try:
            if isinstance(amount, str) and ',' in amount:
                amount = amount.replace(',', '.')
            elif amount == '':
                amount = 1
            amount = float(amount)
        except:
            amount = None
        try:
            if isinstance(refuel, str) and ',' in refuel:
                refuel = refuel.replace(',', '.')
            refuel = float(refuel)
        except:
            refuel = None
        try:
            if isinstance(other, str) and ',' in other:
                other = other.replace(',', '.')
            other = float(other)
        except:
            other = None
        try:
            if isinstance(recharges, str) and ',' in recharges:
                recharges = recharges.replace(',', '.')
            recharges = float(recharges)
        except:
            recharges = None
        try:
            km = int(km)
        except:
            km = None

        ppu = round(float(brutto) / float(amount), 3)
        if file:
            vals = [current_id_users, id_all_cars, data, alim_type, brutto, amount, refuel, other, recharges, ppu, km,
                    comment, file, file_name, provider]
        else:
            vals = [current_id_users, id_all_cars, data, alim_type, brutto, amount, refuel, other, recharges, ppu, km,
                    comment, provider]

        self.alimentari.addNewRow(cols, tuple(vals))

    def create_sql_table(self, table_name):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        masina_sql = os.path.join(os.path.dirname(__file__), 'static', 'sql',
                                  'auto_template.sql')

        # masina_sql = r'static\sql\auto.sql'
        mysql_rm.DataBase(self.conf.credentials).createTableFromFile(masina_sql, table_name)

    def last_records_for_plot(self, delta_mths=12):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        plot_lr = {}
        start_date = date.today() - relativedelta(months=+delta_mths)
        # print('start_date', start_date)
        last_rec_tab_head = list(self.last_records[0])
        # print(last_rec_tab_head)
        for lr in self.last_records[1:]:
            # print(lr)
            ali_date = lr[last_rec_tab_head.index('data')]
            recharge_type = lr[last_rec_tab_head.index('type')]
            brutto = lr[last_rec_tab_head.index('brutto')]
            if ali_date > start_date:
                # print('BINGOOOOO')
                total_interval = date.today() - start_date
                interval_since_lr = date.today() - ali_date
                # print(total_interval, interval_since_lr)
                # print(recharge_type, interval_since_lr/total_interval)
                label = '{}_{}_{}'.format(recharge_type, ali_date, brutto)
                plot_lr[label] = interval_since_lr/total_interval
        return plot_lr


def main():
    script_start_time = time.time()
    selectedStartDate = datetime(2024, 7, 15, 0, 0, 0)
    selectedEndDate = datetime(2025, 7, 15, 0, 0, 0)

    # auto_app = AutoApp(rappmysql.ini_masina, 1)
    # print(auto_app.masini)
    app_masina = Masina(rappmysql.ini_masina, 1, 14)
    # alimentari = app_masina.get_alimentari_for_interval_type(selectedStartDate, selectedEndDate, None)
    # for i in alimentari:
    #     print(i)
    # print(len(alimentari))
    # print(app_masina.tot_benzina)
    # print(app_masina.tot_electric)
    print(app_masina.monthly_benzina)
    # print(app_masina.monthly_electric)
    # print(app_masina.dict_totals)
    # print(app_masina.bkp_table_totals)
    # print(app_masina.monthly)
    # dict_last_months = app_masina.dict_last_months()
    # for interval, values in dict_last_months.items():
    #     print(interval)
    #     print(values)
    #     print(type(values))
    # app_masina.last_records_for_plot()
    # print()
    # print(app_masina.table_totals)
    # print('app_masina.last_records')
    # print(app_masina.last_records)

    scrip_end_time = time.time()
    duration = scrip_end_time - script_start_time
    duration = time.strftime("%H:%M:%S", time.gmtime(duration))
    print('run time: {}'.format(duration))


if __name__ == '__main__':
    main()



from configparser import ConfigParser
import sys


class Config:
    def __init__(self, fileName):
        #print('Module: {}, {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.fileName = fileName
        if len(self.sections) == 1:
            self.credentials = 0

    @property
    def credentials(self):
        #print('Module: {}, {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        parser = ConfigParser(empty_lines_in_values=False)
        parser.read(self.fileName)
        sections = parser.sections()
        sect = sections[self.sec_no]
        params = dict(parser.items(sect))
        return params

    @credentials.setter
    def credentials(self, sec_no):
        #print('Module: {}, {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.sec_no = sec_no

    @property
    def sections(self):
        #print('Module: {}, {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        parser = ConfigParser(empty_lines_in_values=False)
        parser.read(self.fileName)
        sections = parser.sections()
        return sections

    @property
    def db_type(self):
        #print('Module: {}, {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        parser = ConfigParser(empty_lines_in_values=False)
        parser.read(self.fileName)
        sections = parser.sections()
        type = sections[self.sec_no].split('_')[0]
        return type
